"""
API interface to T0AST DB specific to BlockIAuditor.
The main purpose of the module is combine T0AST DB queries 
in a way it should happen in one transaction

"""

__revision__ = "$Id: BlockAuditorAPI.py,v 1.44 2010/07/14 14:05:40 hufnagel Exp $"
__version__ = "$Revision: 1.44 $"

import logging

from WMCore.WMBS.File import File as WMBSFile

from T0 import Globals
from T0.State.Database.Reader.ListFiles import listFileIDsByBlockID
from T0.State.Database.Writer.InsertBlock import updateBlockStatusByID
from T0.State.Database.Writer.InsertBlock import updateBlockMigrateStatusByID
from T0.State.Database.Writer.InsertBlock import updateExportEndTime

def updateBlockMigrateStatus(dbConn, blockList, status):
    """
    _updateBlockMigrateStatus_
    
    To do: Is it better here or move to BlockAuditor component
    """
    dbConn.beginTransaction()
    updateBlockMigrateStatusByID(dbConn, blockList, status)
    dbConn.commit()

def updateBlockFileStatusToExported(dbConn, block, locationList, expressFlag=False):
    # update export Subscription for all the files in the block
    dbConn.beginTransaction()
    if type(locationList) == str:
        locationList = [locationList]
        
    fileIDs = listFileIDsByBlockID(dbConn, block["BLOCK_ID"])
    
    logging.debug("Marking files in the Block :[%s] for Deletion\nFile IDs:%s" 
                  % (block["BLOCK_ID"], fileIDs))                
    if not fileIDs:
        logging.error("There should be at least one file for a given block")
        # should raise an error
        return False
    
    # update export Subscription for all the files in the block
    if expressFlag:
        exportSub = Globals.retrieveExpressTransferSubscription()
    else:
        exportSub = Globals.retrieveTransferSubscription()
        
    wmbsFileList = []
    for fileID in fileIDs:
        wmbsFileList.append( WMBSFile(id = fileID) )

    exportSub.completeFiles(wmbsFileList)
    updateBlockStatusByID(dbConn, block, "Exported")
    updateExportEndTime(dbConn, block["BLOCK_ID"])
    dbConn.commit()

    return fileIDs
