#!/usr/bin/env python
"""
_ConfDBAPI_

"""
__version__ = "$Revision: 1.13 $"
__revision__ = "$Id: ConfDBAPI.py,v 1.13 2010/10/03 16:08:53 hufnagel Exp $"


import logging

from ProdAgentCore.Configuration import loadProdAgentConfiguration
from T0.GenericTier0 import Tier0DB


class ConfDB:
    """
    _ConfDB_

    API to access ConfDB information directly from the database

    """
    def __call__(self, configName):
        """
        _operator()_

        Retrieve the stream/dataset/path mapping for a given config name

        """
        # temporary shortcut for the Tier0 Relval
        if ( configName == 'Tier0Relval' ):
            streamsToDatasets = { 'A' : [ 'RelvalDataset1', 'RelvalDataset2' ] }
            datasetsToPaths = { 'RelvalDataset1' : [ 'HLT_Jet30' ],
                                'RelvalDataset2' : [ 'HLT_Jet110' ] }
            return { "Streams" : streamsToDatasets,
                     "Datasets": datasetsToPaths,
                     "Process" : 'HLT',
                     }

        paConfig = loadProdAgentConfiguration()
        confDBConfig = paConfig.getConfig("ConfDB")

        confDBConn = Tier0DB.Tier0DB(confDBConfig,
                                     manageGlobal = False)

        confDBConn.connect()

        sqlQuery = """SELECT d.streamlabel, c.datasetLabel, b.name, h.processname FROM PathStreamDatasetAssoc a 
                      INNER Join Paths b
                      ON a.pathID = b.pathID
                      INNER JOIN PrimaryDatasets c
                      ON a.datasetID = c.datasetID
                      INNER JOIN Streams d 
                      ON a.streamId = d.streamID
                      INNER JOIN ECStreamAssoc  e
                      ON d.streamid = e.STREAMID
                      INNER JOIN EventContents f
                      ON e.eventContentId = f.eventContentId
                      INNER JOIN ConfigurationContentAssoc g 
                      ON g.eventContentId = f.eventContentId
                      INNER JOIN Configurations h
                      ON g.configID = h.configID
                      WHERE h.configdescriptor = :p_1
                      AND d.fracToDisk > 0"""

        bindVars = {"p_1": configName}
        confDBConn.execute(sqlQuery, bindVars, useTransaction = False)
        results = confDBConn.fetchall()

        processName = None
        mapping = {}

        for result in results:

            stream = result[0]
            dataset = result[1]
            path = result[2]
            processName = result[3]
    
            if mapping.has_key(stream):
                if mapping[stream].has_key(dataset):
                    mapping[stream][dataset].append(path)
                else:
                    mapping[stream][dataset] = [path]
            else:
                mapping[stream] = { dataset : [path] }

        if len(mapping) == 0:
            errorMsg = "Couldn't extract stream-dataset-path mapping."
            raise Exception, errorMsg

        logging.info("Mapping extracted for config %s" % configName)

        return [ processName, mapping ]
        
