import urllib
import logging
import os
import pwd

from WMCore.Services.Service import Service

try:
    # Python 2.6
    import json
except ImportError:
    # Prior to 2.6 requires simplejson
    import simplejson as json

class Tier0Monitor(Service):

    """
    API for dealing with retrieving information from PhEDEx DataService
    """

    def __init__(self, dict = {}, responseType = "json", secure = False):
        """
        responseType will be either xml or json
        """
        self.responseType = responseType.lower()

        if not dict.has_key('endpoint'):
            dict['endpoint'] = "%scmsweb.cern.ch/tier0Monintor/" % \
                                ((secure and "https://" or "http://"))
        if dict.has_key('cachepath'):
            pass
        elif os.getenv('TIER0_MONITOR_CACHE_DIR'):
            dict['cachepath'] = os.getenv('TIER0_MONITOR_CACHE_DIR') + '/.tier0_monitotcache'
        elif os.getenv('HOME'):
            dict['cachepath'] = os.getenv('HOME') + '/.tier0_monitotcache'
        else:
            dict['cachepath'] = '/tmp/tier0_monitor_' + pwd.getpwuid(os.getuid())[0]
        if not os.path.isdir(dict['cachepath']):
            os.makedirs(dict['cachepath'])
        if 'logger' not in dict.keys():
            logging.basicConfig(level = logging.DEBUG,
                    format = '%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                    datefmt = '%m-%d %H:%M',
                    filename = dict['cachepath'] + '/phedexdbjsonparser.log',
                    filemode = 'w')
            dict['logger'] = logging.getLogger('PhEDExParser')

        Service.__init__(self, dict)

    def _getResult(self, callname, file = 'result', clearCache = True, 
                   args = None, verb="GET"):
        """
        _getResult_

        retrieve JSON/XML formatted information given the service name and the
        argument dictionaries

        TODO: Probably want to move this up into Service
        """
        result = ''
        if clearCache:
            self.clearCache(file, args)
        try:
            # overwrite original self['method']
            # this is only place used self['method'], it is safe to overwrite
            # If that changes keep the reset to original self['method']   
            self["method"] = verb     
            f = self.refreshCache(file, callname, args)
            result = f.read()
            f.close()
            
        except IOError, ex:
            raise RuntimeError("URL not available: %s" % callname)

        if self.responseType == "json":
            decoder = json.JSONDecoder()
            return decoder.decode(result)

        return result

    def getRuns(self):

        """
        _getRuns_
    
        """

        callname = 'runs'
        
        return self._getResult(callname)

    def getRunsByStatus(self):
        
        callname = 'runsbystatus'
        return self._getResult(callname)
        
    def getRunsByRecoStatus(self):
        
        callname = 'runsbyrecostatus'
        return self._getResult(callname)
    
    def getRecoStatusByRun(self, run):
        
        callname = 'recostatusbyrun'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRecoTimeLeft(self):
        
        callname = 'recotimeleft'
        return self._getResult(callname)
    
    def getRecoTimeLeftByRun(self, run):
        
        callname = 'recotimeleftbyrun'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
        # tom specific DAS
    
    def getRepackEventstByRun(self, run):
        
        callname = 'repackevents'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
        
    def getRecoEventstByRun(self, run):
        
        callname = 'recoevents'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getAlcaEventstByRun(self, run):
        
        callname = 'alcaevents'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRecoFileSizeByRun(self, run):
        
        callname = 'recofilesize'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getAlcaFileSizeByRun(self, run):
        
        callname = 'alcafilesize'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getAlcaJobTimesByRun(self, run):
        
        callname = 'alcajobtimes'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getMergeJobTimesByRun(self, run):
        
        callname = 'mergejobtimes'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getEventsPerDatasetByRun(self, run):
        
        callname = 'eventsperdataset'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getEventsPerLumiByRun(self, run):
        
        callname = 'eventsperlumisection'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getEventsPerStreamByRun(self, run):
        
        callname = 'eventsperstream'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getEventsPerStreamAndDatasetByRun(self, run):
        
        callname = 'eventsperstreamanddataset'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getExpressJobsByRun(self, run):
        
        callname = 'expressjobs'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRepackJobsByRun(self, run):
        
        callname = 'repackjobs'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getExpressLatencyByRun(self, run):
        
        callname = 'expresslatency'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
        #TODO fix server side sql
    def getFailedJobCountByRun(self, fileset, lastUpdate):
        
        callname = 'failedjobcount'
        args = {}
        args['fileset'] = fileset
        args['last_update'] = lastUpdate
        return self._getResult(callname, args=args)
    
    def getFailedJobCountByRun(self, fileset, lastUpdate):
        
        callname = 'failedjobcount'
        args = {}
        args['fileset'] = fileset
        args['last_update'] = lastUpdate
        return self._getResult(callname, args=args)
        
    def getFailedExpressJobs(self, definitionTime):
        
        callname = 'getfailedexpressjob'
        args = {}
        args['definition_time'] = definitionTime
        return self._getResult(callname, args=args)   
        
    def getFailedRepackJobs(self, definitionTime):
        
        callname = 'getfailedrepackjob'
        args = {}
        args['definition_time'] = definitionTime
        return self._getResult(callname, args=args)   
        
    def getHeartBeats(self):

        """
        _getHeartBeats_
    
        """

        callname = 'heartbeats'
        
        return self._getResult(callname)

    def getIncompleteRuns(self):

        """
        _getIncompleteRuns_
    
        """

        callname = 'incompleteruns'
        
        return self._getResult(callname)

    def getInsertTimeVsRate1Hour(self):

        """
        _getInsertTimeVsRate1Hour_
    
        """

        callname = 'inserttimevsrate1hour'
        
        return self._getResult(callname)
    
    def getInsertTimeVsRate5Hours(self):

        """
        _getInsertTimeVsRate5Hours_
    
        """

        callname = 'inserttimevsrate5hours'
        
        return self._getResult(callname)

    def getIsRunComplete(self, run):
        
        callname = 'isruncomplete'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getJobsByState(self, run, filesetName, workflowName):
        
        callname = 'jobsbystate'
        args = {}
        args['run'] = run
        args['fileset_name'] = filesetName
        args['workflow_name'] = workflowName
        return self._getResult(callname, args=args)
    
    def getMergeFilesForRun(self, run):
        
        callname = 'mergefilesforrun'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getMergeFileStatus(self, run, dataTier):
        
        callname = 'mergefilestatus'
        args = {}
        args['run'] = run
        args['data_tier'] = dataTier
        return self._getResult(callname, args=args)
    
    def getProcessingFileStatus(self, run, workflow):
        
        callname = 'processingfilestatus'
        args = {}
        args['run'] = run
        args['workflow'] = workflow
        return self._getResult(callname, args=args)
    
    def getRawFileSizeByRun(self, run):
        
        callname = 'rawfilesize'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRepackFileSizeAlarmByRun(self, run):
        
        callname = 'repackfilesizealarm'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRecoFileSizeAlarmByRun(self, run):
        
        callname = 'recofilesizealarm'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
        
    def getRepackFileSizePerDataset(self, run):
        
        callname = 'repackfilesizeperdataset'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRecoFileSizePerDataset(self, run):
        
        callname = 'recofilesizeperdataset'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRepackJobTimesByRun(self, run):
        
        callname = 'repackjobtimes'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRecoJobTimesByRun(self, run):
        
        callname = 'recojobtimes'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
        
    def getMergeJobTimesByRun(self, run):
        
        callname = 'mergejobtimes'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRepackJobStatusByRun(self, run):
        
        callname = 'repackjobstatus'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRunDetail(self, run):
        
        callname = 'rundetail'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getRunTable(self):
        
        callname = 'runtable'  
        return self._getResult(callname)
    
    def getStreamersToSplit(self, run):
        
        callname = 'streamerstosplit'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getStreamersTotals(self, run):
        
        callname = 'streamertotals'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)
    
    def getUnmergedFilesForRun(self, run):
        
        callname = 'unmergedfilesforrun'
        args = {}
        args['run'] = run
        return self._getResult(callname, args=args)