#!/usr/bin/env python
"""
_Information on the Express configuration optionally filtered by run and stream_

T0Mon: this class is used to print the Express Config Table in the run pages.
"""

from WMCore.Database.DBFormatter import DBFormatter

class ExpressStreamConfig(DBFormatter):
   
   sql = """SELECT  st.name AS stream,
                   ec.run_id AS run_id,
                   ec.splitinprocessing AS splitinprocessing,
                   ec.proc_version AS proc_version,
                   ec.processing_config_url AS config_url,
                   ec.global_tag AS global_tag,
                   ec.alcamerge_config_url AS alcamerge_config_url,
		           cv.name as expversion,
		           es.scenario as scenario
            FROM express_config ec
            INNER JOIN stream st ON (st.id = ec.stream_id)
            INNER JOIN run r ON  (r.run_id = ec.run_id)
     	    INNER JOIN cmssw_version cv ON (cv.id= r.express_version)
            INNER JOIN stream_special_dataset_assoc  sd ON (sd.stream_id =ec.stream_id)
            INNER JOIN scenario_config sc ON (ec.run_id = sc.run_id   AND sc.primary_dataset_id = sd.primary_dataset_id)
            INNER JOIN event_scenario es ON (sc.scenario_id = es.id)
            WHERE %s %s
            ORDER BY stream"""

   sqlWhereWithRun="ec.run_id = :run"
   sqlWhereWithoutRun="ec.run_id = (select max(run_id) from express_config)"
   sqlWhereOptionStream=" AND st.name = :stream"
  
   
   def execute(self, run = None, stream = None,
                     conn = None, transaction = False):
        """
        _execute_

        Execute the SQL 
        """
        if run and stream:

           self.sql = self.sql % (self.sqlWhereWithRun,self.sqlWhereOptionStream)

           if type(run) != list:
              run = [run]
           if type(stream) != list:
              stream = [stream]

           bindVars = []
           for runElem in run:
              for streamElem in stream:
                 bindVars.append( { "run" : runElem,
                                    "stream" : streamElem } )

        elif run:

           self.sql = self.sql % (self.sqlWhereWithRun, '')

           if type(run) != list:
              run = [run]

           bindVars = []
           for runElem in run:
              bindVars.append( { "run" : runElem } )

        elif stream:

           self.sql = self.sql % (self.sqlWhereWithoutRun, self.sqlWhereOptionStream)

           if type(stream) != list:
              stream = [stream]
              
           bindVars = []
           for streamElem in stream:
                 bindVars.append( { "stream" : streamElem } )

        else:

           self.sql = self.sql % (self.sqlWhereWithoutRun, '')
           bindVars = {}
           
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result) 
