#!/usr/bin/env python
"""
_EventsPerDataset_

Monitoring DAO classes for Runs in WMBS
"""
__all__ = []
__revision__ = "$Id: EventsPerDataset.py,v 1.3 2010/04/07 13:11:52 hufnagel Exp $"
__version__ = "$Revision: 1.3 $"

from WMCore.Database.DBFormatter import DBFormatter

from T0 import Globals

class EventsPerDataset(DBFormatter):

    subscription = Globals.retrieveDBSUploadSubscription()
    
    sql = """SELECT primary_dataset.name AS name,
                    SUM(wmbs_file_details.events) AS total_events
             FROM wmbs_fileset_files
             INNER JOIN wmbs_subscription ON
               wmbs_subscription.fileset = wmbs_fileset_files.fileset AND
               wmbs_subscription.id = %d
             INNER JOIN wmbs_file_runlumi_map ON
               wmbs_file_runlumi_map.fileid = wmbs_fileset_files.fileid AND
               wmbs_file_runlumi_map.run = :run
             INNER JOIN wmbs_file_dataset_path_assoc ON
               wmbs_file_dataset_path_assoc.file_id = wmbs_fileset_files.fileid
             INNER JOIN dataset_path ON
               dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
             INNER JOIN primary_dataset ON
               primary_dataset.id = dataset_path.primary_dataset
             INNER JOIN data_tier ON
               data_tier.id = dataset_path.data_tier AND
               data_tier.name = 'RAW'
             INNER JOIN wmbs_file_details ON
               wmbs_file_details.id = wmbs_fileset_files.fileid
             GROUP BY primary_dataset.name
             """ % subscription['id']

    def execute(self, run, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL and then format and return the result.
        """
        bindVars = {"run": run}
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result)
