#!/usr/bin/env python
"""
_EventsPerStream_

Monitoring DAO classes for Runs in WMBS
"""
__all__ = []
__revision__ = "$Id: EventsPerStream.py,v 1.1 2010/03/17 15:22:47 sryu Exp $"
__version__ = "$Revision: 1.1 $"

from WMCore.Database.DBFormatter import DBFormatter

class EventsPerStream(DBFormatter):
    
    sql = """SELECT SUM(events) as total_events, name AS streamname FROM 
               (SELECT distinct streamer_id, events, stream.name AS name FROM streamer 
               INNER JOIN stream on streamer.stream_id = stream.id
               INNER JOIN dataset_run_stream_assoc ON stream.id = dataset_run_stream_assoc.stream_id
               WHERE streamer.run_id = :run) GROUP BY name"""
             
    def execute(self, run, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL for the given reco_status list and then format and return
        the result.
        """
        bindVars = {"run": run}
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result)