#!/usr/bin/env python
"""
_EventsPerStreamAndDataset_

Monitoring DAO classes for Runs in WMBS
"""
__all__ = []
__revision__ = "$Id: EventsPerStreamAndDataset.py,v 1.1 2010/03/17 15:22:47 sryu Exp $"
__version__ = "$Revision: 1.1 $"

from WMCore.Database.DBFormatter import DBFormatter

class EventsPerStreamAndDataset(DBFormatter):
    
    sql = """SELECT SUM(events)AS total_events, dataset AS datasetname, name AS streamname FROM
               (SELECT distinct wmbs_file_details.id, events, primary_dataset.name AS dataset, stream.name as name FROM wmbs_file_details
             INNER JOIN wmbs_file_dataset_path_assoc ON wmbs_file_dataset_path_assoc.file_id = wmbs_file_details.id
             INNER JOIN dataset_path ON wmbs_file_dataset_path_assoc.dataset_path_id = dataset_path.id 
             INNER JOIN data_tier ON dataset_path.data_tier = data_tier.id 
             INNER JOIN primary_dataset ON dataset_path.primary_dataset = primary_dataset.id
             INNER JOIN wmbs_file_runlumi_map ON wmbs_file_details.id = wmbs_file_runlumi_map.fileid
             INNER JOIN wmbs_fileset_files ON wmbs_fileset_files.fileid = wmbs_file_details.id
             INNER JOIN wmbs_fileset ON wmbs_fileset_files.fileset = wmbs_fileset.id
             INNER JOIN dataset_run_stream_assoc ON
                   run = dataset_run_stream_assoc.run_id AND
                   dataset_path.primary_dataset = dataset_run_stream_assoc.primary_dataset_id
             INNER JOIN stream on dataset_run_stream_assoc.stream_id = stream.id
             WHERE run = :run AND data_tier.name = 'RAW' AND wmbs_fileset.name = 'DBSUploadable')
             GROUP BY name, dataset"""
             
    def execute(self, run, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL for the given reco_status list and then format and return
        the result.
        """
        bindVars = {"run": run}
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result)