#!/usr/bin/env python
"""
_MergeFileStatus_

Monitoring DAO classes for Runs in WMBS
"""
__all__ = []
__revision__ = "$Id: MergeFileStatus.py,v 1.2 2010/04/07 13:11:52 hufnagel Exp $"
__version__ = "$Revision: 1.2 $"

from WMCore.Database.DBFormatter import DBFormatter

from T0 import Globals

class MergeFileStatus(DBFormatter):

    subscription = Globals.retrieveMergeSubscription()
    
    sql = """SELECT COUNT(wmbs_sub_files_acquired.fileid) AS Acquired,
                    COUNT(wmbs_sub_files_complete.fileid) AS Completed, 
                    COUNT(wmbs_sub_files_failed.fileid) AS Failed      
             FROM wmbs_fileset_files
             INNER JOIN wmbs_subscription ON
                 wmbs_subscription.fileset = wmbs_fileset_files.fileset AND
                 wmbs_subscription.id = %d
             INNER JOIN wmbs_file_runlumi_map ON
               wmbs_file_runlumi_map.fileid = wmbs_fileset_files.id AND
               wmbs_file_runlumi_map.run = :run
             INNER JOIN wmbs_file_dataset_path_assoc ON
               wmbs_file_dataset_path_assoc.file_id = wmbs_fileset_files.id 
             INNER JOIN dataset_path ON
               dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
             INNER JOIN data_tier ON
               data_tier.id = dataset_path.data_tier AND
               data_tier.name = :datatier
             LEFT OUTER JOIN wmbs_sub_files_acquired ON
               wmbs_sub_files_acquired.fileid  = wmbs_fileset_files.id AND
               wmbs_sub_files_failed.subscription = %d
             LEFT OUTER JOIN wmbs_sub_files_complete ON
               wmbs_sub_files_complete.fileid  = wmbs_fileset_files.id AND
               wmbs_sub_files_complete.subscription = %d
             LEFT OUTER JOIN wmbs_sub_files_failed ON
               wmbs_sub_files_failed.fileid  = wmbs_fileset_files.id AND
               wmbs_sub_files_failed.subscription = %d
             """ % (subscription['id'],
                    subscription['id'],
                    subscription['id'],
                    subscription['id'])
                        
    def execute(self, run, datatier, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL and then format and return the result.
        
        datatier: RAW RECO ALCARECO
        """
        bindVars = {"run": run, "datatier": datatier}
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result)
