#!/usr/bin/env python
"""
_IsRunComplete_

Monitoring DAO classes for Runs in WMBS
"""
__all__ = []
__revision__ = "$Id: FailedJobCount.py,v 1.1 2010/03/17 15:22:45 sryu Exp $"
__version__ = "$Revision: 1.1 $"

from WMCore.Database.DBFormatter import DBFormatter

class FailedJobCount(DBFormatter):
    """
    The type of jobs are: 
    Merge Jobs -> "Mergeable"
    MergePack Jobs -> ExpressMergeable"
    Reco Jobs -> "Reconstructable"
    Alca Jobs -> "AlcaSkimmable"
    """
    
    #TO: check what else is needed for return item
    sql = """ SELECT wmbs_job.id AS job_id, wmbs_job.submission_time
                 AS definition_time FROM wmbs_job
                INNER JOIN wmbs_jobgroup ON wmbs_job.jobgroup = wmbs_jobgroup.id 
                INNER JOIN wmbs_subscription ON wmbs_jobgroup.subscription = wmbs_subscription.id 
                INNER JOIN wmbs_group_job_failed ON wmbs_job.id = wmbs_group_job_failed.job 
              WHERE wmbs_subscription.fileset = (SELECT id FROM wmbs_fileset WHERE name = :fileset)
                AND wmbs_job.last_update > :last_update
              ORDER by job_id"""
             
    def execute(self, fileset, last_update, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL for the given reco_status list and then format and return
        the result.
        """
        bindVars = {"fileset": fileset, "last_update": last_update}
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result)