#!/usr/bin/env python
"""
_IsRunComplete_

Monitoring DAO classes for Runs in WMBS
"""
__all__ = []
__revision__ = "$Id: JobsByState.py,v 1.1 2010/03/17 15:22:45 sryu Exp $"
__version__ = "$Revision: 1.1 $"

from WMCore.Database.DBFormatter import DBFormatter

class JobsByState(DBFormatter):
    
    """
    * find job status for a specific run
    * This query will return three columns: ACQUIRED, COMPLETED and FAILED.  Acquired jobs are jobs that have been injected into
    * the batch system, completed and successful jobs while failed are failed jobs.  The jobs will have a total for the jobs in
    * that particular state for the run.
    * The type of jobs are: 
    * Merge Jobs -> "Mergeable"
    * MergePack Jobs -> ExpressMergeable"
    * Reco Jobs -> "Reconstructable"
    * Alca Jobs -> "AlcaSkimmable"
    * NOTE: repack jobs are queried with a separate query
    """
    
    sql = """SELECT wmbs_job_state.name AS job_state, count(DISTINCT wmbs_job.id) AS numJobs 
             FROM wmbs_job
              INNER JOIN wmbs_job_state ON wmbs_job_state.id = wmbs_job.state
              INNER JOIN wmbs_job_assoc ON wmbs_job_assoc.job = wmbs_job.id 
              INNER JOIN wmbs_file_runlumi_map ON wmbs_file_runlumi_map.FILEID = wmbs_job_assoc.FILEID
              INNER JOIN wmbs_jobgroup ON wmbs_jobgroup.id = wmbs_job.jobgroup 
              INNER JOIN wmbs_subscription ON wmbs_subscription.id = wmbs_jobgroup.subscription 
              INNER JOIN wmbs_fileset ON wmbs_fileset.id = wmbs_subscription.fileset 
              INNER JOIN wmbs_workflow ON wmbs_workflow.id = wmbs_subscription.workflow  
             WHERE wmbs_fileset.name = :fileset_name AND wmbs_workflow.name = :workflow_name 
                   AND wmbs_file_runlumi_map.run = :run
             GROUP BY wmbs_job_state.name"""
             
    def execute(self, run, fileset_name, workflow_name, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL for the given reco_status list and then format and return
        the result.
        """
        bindVars = {"run": run, "fileset_name": fileset_name, "workflow_name": workflow_name}
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result)