#!/usr/bin/env python
"""
_ListLumiRates_

BEHOLD: Dave Mason's lumi rate query.
"""

__revision__ = "$Id: ListLumiRates.py,v 1.1 2010/03/17 15:22:50 sryu Exp $"
__version__ = "$Revision: 1.1 $"

import time
from WMCore.Database.DBFormatter import DBFormatter

class ListLumiRates(DBFormatter):
    sql = """SELECT run_id,
                    lumi_id,
                    stream.name AS stream_name,
                    MIN(insert_time) AS start_time,
                    SUM(events) AS total_events,
                    SUM(events) / 23 AS event_rate,
                    SUM(filesize) / 1024 / 1024 AS data_size,
                    COUNT(lfn) AS num_files,
                    SUM(filesize) / SUM(events) AS event_size,
                    MAX(insert_time) - MIN(insert_time) AS lumi_length
               FROM streamer
               INNER JOIN stream ON
                 streamer.stream_id = stream.id
             WHERE streamer.insert_time > :last_streamer
             GROUP BY run_id, stream.name, lumi_id
             ORDER BY run_id DESC, stream.name, lumi_id DESC"""

    def format(self, results):
        """
        _format_

        Format the floating point values returned from the query here so I
        don't have to do it in JavaScript.
        """
        results = DBFormatter.formatDict(self, results)

        for result in results:
            result["event_rate"] = int(result["event_rate"])
            result["data_size"] = "%.2f" % result["data_size"]
            result["event_size"] = int(result["event_size"])

        return results

    def execute(self, interval = 7200, conn = None, transaction = False):
        """
        _execute_

        Determine the display interval and run the query.
        """
        lastStreamer = int(time.time()) - int(interval)
        result = self.dbi.processData(self.sql, {"last_streamer": lastStreamer},
                                      conn = conn, transaction = transaction)
        return self.format(result)
