#!/usr/bin/env python
"""
_RecoStartTimeLeft_

Monitoring DAO classes for Runs in Tier0
"""
__all__ = []
__revision__ = "$Id: RecoStartTimeLeft.py,v 1.1 2010/03/17 15:22:46 sryu Exp $"
__version__ = "$Revision: 1.1 $"

import time
from WMCore.Database.DBFormatter import DBFormatter
from T0.DAS.GlobalConstants import getRecoTimeOut

class RecoStartTimeLeft(DBFormatter):

    sql = """ SELECT run.run_id, MIN(st.insert_time) FROM streamer st
                  INNER JOIN run_stream_style_assoc rss ON rss.run_id = st.run_id AND rss.stream_id = st.stream_id
                  INNER JOIN processing_style ps ON ps.id = rss.style_id
                  INNER JOIN run ON run.run_id = st.run_id
                  INNER JOIN run_status ON run.run_status = run_status.id
                  WHERE run.reco_started = 0 AND ps.name = 'Bulk'
                  GROUP BY run.run_id """
                  
    def execute(self, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL  then format and return
        the result.
        """
        results = self.dbi.processData(self.sql, conn = conn, transaction = transaction)
        results = self.format(results)
        formattedResults = []
        if len(results) == 0:
            formattedResult = {}
            formattedResult["run"] = "All runs are reco started"
            formattedResult["reco_start_time_left"] = "Not available"
            formattedResults.append(formattedResult)
        else:
            recoTimeOut = getRecoTimeOut()
            # recoTimeOut - current time - insert time
            for row in results:
                formattedResult = {}
                formattedResult["run"] = row[0]
                formattedResult["reco_start_time_left"] = recoTimeOut - (int(time.time()) - row[1])
                formattedResults.append(formattedResult)
                
        return formattedResults