#!/usr/bin/env python
"""
_RecoStartTimeLeft_

Monitoring DAO classes for Runs in WMBS
"""
__all__ = []
__revision__ = "$Id: RecoStartTimeLeftByRun.py,v 1.1 2010/03/17 15:22:46 sryu Exp $"
__version__ = "$Revision: 1.1 $"

from WMCore.Database.DBFormatter import DBFormatter
from T0.DAS.GlobalConstants import getRecoTimeOut
import time

class RecoStartTimeLeftByRun(DBFormatter):
    
    sql = """ SELECT run.run_id, MIN(st.insert_time) FROM streamer st
                  INNER JOIN run_stream_style_assoc rss ON rss.run_id = st.run_id AND rss.stream_id = st.stream_id
                  INNER JOIN processing_style ps ON ps.id = rss.style_id
                  INNER JOIN run ON run.run_id = st.run_id
                  INNER JOIN run_status ON run.run_status = run_status.id
                  WHERE run.reco_started = 0 AND ps.name = 'Bulk' and run.run_id = :run 
                  GROUP BY run.run_id """
             
    def execute(self, run, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL for the given reco_status list and then format and return
        the result.
        """
        bindVars = {"run": run}
        results = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        results = self.format(results)
        formattedResult = {}
        
        if len(results) == 0:
            formattedResult["run"] = run
            formattedResult["reco_start_time_left"] = "Reco started: Not available"
        else:
            recoTimeOut = getRecoTimeOut()
            # recoTimeOut - current time - insert time
            formattedResult["run"] = results[0][0]
            formattedResult["Treco_start_time_left"] = recoTimeOut - (int(time.time()) - results[0][1])
        return formattedResult