
from WMCore.HTTPFrontEnd.WorkQueue.Services.ServiceInterface import ServiceInterface
from WMCore.DAOFactory import DAOFactory

class Tier0TomService(ServiceInterface):
    """
    _Tier0TomService_
    
    Provide the tier0 REST services for tom monitoring.
    
    TODO: this should be combined with Tier0Mon Queries and maybe organized in different way.
    i.e. Tier0JobService, Tier0FileService etc.  
    """
    
    def register(self):
        """
        """
        #set default settings for dao: this shouldn't overwritten from  other Plugins.
        self.model.daofactory = DAOFactory(package = "T0.DAS.Database",
                                     logger = self.model,
                                     dbinterface = self.model.dbi)
        
        # tom specific DAS
        self.model.addDAO('GET', 'repackevents', 'Events.RepackEvents', args=['run'])
        self.model.addDAO('GET', 'recoevents', 'Events.RecoEvents', args=['run'])
        self.model.addDAO('GET', 'alcaevents', 'Events.AlcaEvents', args=['run'])
        
        self.model.addDAO('GET', 'recofilesize', 'Files.RecoFileSize', args=['run'])
        self.model.addDAO('GET', 'alcafilesize', 'Files.AlcaFileSize', args=['run'])
        
        self.model.addDAO('GET', 'eventsperdataset', 'Events.EventsPerDataset', args=['run'])
        self.model.addDAO('GET', 'eventsperlumisection', 'Events.EventsPerLumisection', args=['run']) 
        self.model.addDAO('GET', 'eventsperstream', 'Events.EventsPerStream', args=['run'])
        self.model.addDAO('GET', 'eventsperstreamanddataset', 'Events.EventsPerStreamAndDataset', args=['run'])
        self.model.addDAO('GET', 'expressjobs', 'Jobs.ExpressJobs', args=['run'])
        self.model.addDAO('GET', 'repackjobs', 'Jobs.RepackJobs', args=['run'])
        self.model.addDAO('GET', 'expresslatency', 'Timing.ExpressLatency', args=['run'])
        
        #TODO fix sql
        self.model.addDAO('GET', 'failedjobcount', 'Jobs.FailedJobCount', args=['fileset', 'last_update'])
        
        self.model.addDAO('GET', 'getfailedexpressjob', 'Jobs.GetFailedExpressJob', args=['definition_time'])
        self.model.addDAO('GET', 'getfailedrepackjob', 'Jobs.GetFailedRepackJob', args=['definition_time'])
        self.model.addDAO('GET', 'heartbeats', 'Components.HeartBeats')
        self.model.addDAO('GET', 'incompleteruns', 'Runs.IncompleteRuns')
        self.model.addDAO('GET', 'inserttimevsrate1hour', 'Streamers.InsertTimeVsRate1Hour')
        self.model.addDAO('GET', 'inserttimevsrate5hours', 'Streamers.InsertTimeVsRate5Hours')
        self.model.addDAO('GET', 'isruncomplete', 'Runs.IsRunComplete', args=['run'])
        self.model.addDAO('GET', 'jobsbystate', 'Jobs.JobsByState', args=['run', 'fileset_name', 'workflow_name'])
        self.model.addDAO('GET', 'mergefilesforrun', 'Files.MergeFilesForRun', args=['run'])
        self.model.addDAO('GET', 'mergefilestatus', 'Files.MergeFileStatus', args=['run', 'datatier'])
        self.model.addDAO('GET', 'processingfilestatus', 'Files.ProcessingFileStatus', args=['run', 'workflow'])
        self.model.addDAO('GET', 'rawfilesize', 'Files.RawFileSize', args=['run'])
    
        self.model.addDAO('GET', 'repackfilesizealarm', 'Files.RepackFileSizeAlarm', args=['run'])
        self.model.addDAO('GET', 'recofilesizealarm', 'Files.RecoFileSizeAlarm', args=['run'])
        
        self.model.addDAO('GET', 'repackfilesizeperdataset', 'Files.RepackFileSizePerDataset', args=['run'])
        self.model.addDAO('GET', 'recofilesizeperdataset', 'Files.RecoFileSizePerDataset', args=['run'])
        
        #TODO: no creation time and submission time: tracked by couch
        self.model.addDAO('GET', 'alcajobtimes', 'Timing.AlcaJobTimes', args=['run'])
        self.model.addDAO('GET', 'repackjobtimes', 'Timing.RepackJobTimes', args=['run'])
        self.model.addDAO('GET', 'recojobtimes', 'Timing.RecoJobTimes', args=['run'])
        self.model.addDAO('GET', 'mergejobtimes', 'Timing.MergeJobTimes', args=['run'])
        
        self.model.addDAO('GET', 'repackjobstatus', 'Jobs.RepackJobStatus', args=['run'])
        self.model.addDAO('GET', 'rundetail', 'Runs.RunDetail', args=['run'])
        self.model.addDAO('GET', 'runtable', 'Runs.RunTable', args=[])
        
        self.model.addDAO('GET', 'streamerstosplit', 'Streamers.StreamersToSplit', args=['run'])
        self.model.addDAO('GET', 'streamertotals', 'Streamers.StreamerTotals', args=['run'])
        self.model.addDAO('GET', 'unmergedfilesforrun', 'Files.UnmergedFilesForRun', args=['run'])

        self.model.addDAO('GET', 'listfilesoverinterval', 'Files.ListFilesOverInterval',
                    args = ['fileType', 'startTime', 'primaryDataset', 'processedDataset',
                            'dataTier'])
        self.model.addDAO('GET', 'listrecentfiles', 'Files.ListRecentFiles',
                    args = ['fileType', 'period', 'primaryDataset', 'dataTier'])        
        self.model.addDAO('GET', 'listlumirates', 'Lumis.ListLumiRates', args = ['interval'])

