#!/usr/bin/env python
"""
_RuntimeAlcaHarvest_

"""

__revision__ = "$Id: RuntimeAlcaHarvest.py,v 1.3 2010/10/08 16:52:04 hufnagel Exp $"
__version__ = "$Revision: 1.3 $"

import os
import stat
import subprocess
import tempfile
import tarfile
import shutil

from ProdCommon.MCPayloads.JobSpec import JobSpec
from ProdCommon.MCPayloads.WorkflowSpec import WorkflowSpec
from ProdCommon.FwkJobRep.TaskState import TaskState
from ProdCommon.FwkJobRep.FwkJobReport import FwkJobReport
from ProdCommon.FwkJobRep.MergeReports import updateReport


class OfflineAlcaHarvester:
    """
    _OfflineAlcaHarvester_

    Util to trawl through a Framework Job Report to find analysis sqlite files,
    copy them somewhere and upload them to the Offline DropBox

    """
    def __init__(self):
        self.state = TaskState(os.getcwd())
        self.state.loadRunResDB()
        self.config = self.state.configurationDict()
        self.workflowSpecId = self.config['WorkflowSpecID'][0]
        self.jobSpecId = self.config['JobSpecID'][0]
        self.state.loadJobSpecNode()


    def processJobReport(self):
        """
        _readJobReport_

        Process job report, extract sqlite file information, prepare for upload

        """
        print "\n==> Reading job report for sqlite file information.\n"

        workflowSpec = WorkflowSpec()
        workflowSpec.load(os.environ['PRODAGENT_WORKFLOW_SPEC'])

        targetDir = workflowSpec.parameters.get("SqliteTargetDir")
        dropboxHost = workflowSpec.parameters.get("DropBoxHost")
        dropboxValidation = workflowSpec.parameters.get("DropBoxValidation")

        self.toplevelReport = os.path.join(os.environ['PRODAGENT_JOB_DIR'],
                                           "FrameworkJobReport.xml")
        try:
            self.state.loadJobReport()
        except Exception, ex:
            msg = "No Job Report available or could not be read.\n"
            msg += "==> Unable to process analysis files for conditions upload\n"
            print "Creating JobReport by hand..."
            self.state._JobReport = jobRep = FwkJobReport()
            jobRep.name = self.state.taskAttrs['Name']
            jobRep.workflowSpecId = self.state.taskAttrs['WorkflowSpecID']
            jobRep.jobSpecId = self.state.jobSpec.parameters['JobName']
            jobRep.jobType = self.state.taskAttrs['JobType']
            self.failJob(msg)
            return

        jobRep = self.state._JobReport

        # do not upload files if the job failed
        if not jobRep.wasSuccess():
            msg = "FrameworkJobReport says the job has failed."
            msg = "\n==> Not doing anything."
            return

        if len(jobRep.analysisFiles) == 0:
            msg = "AlcaHarvest job did not write any analysis files, nothing to upload.\n"
            self.failJob(msg)

        files2copy = set([])
        files2upload = []
        files2delete = []
        for aFile in jobRep.analysisFiles:

            uuid = subprocess.Popen("uuidgen",stdout=subprocess.PIPE).communicate()[0].strip()

            sqlitefile = aFile['FileName'].replace('sqlite_file:', '', 1)
            # sqlitefile = '/afs/cern.ch/cms/CAF/CMSALCA/ALCA_PROMPT/ALCAPROMPTHarvest-Run147390-StreamExpress-ALCAPROMPT-525802.db'

            files2copy.add(sqlitefile)

            filenamePrefix = "%s@%s" % (aFile['tag'], uuid)
            filenameDB = filenamePrefix + ".db"
            filenameTXT = filenamePrefix + ".txt"
            filenameTAR = filenamePrefix + ".tar.bz2"

            shutil.copy2(sqlitefile, filenameDB)

            # HACK override destDB for validation uploads
            if dropboxValidation != "False":
                aFile['destDB'] = "oracle://cms_orcoff_prep/CMS_COND_BEAMSPOT"

            textoutput = "destDB %s\n" % aFile['destDB']
            textoutput += "tag %s\n" % aFile['tag']
            textoutput += "inputtag %s\n" % aFile['inputtag']
            textoutput += "since\n"
            textoutput += "Timetype %s\n" % aFile['Timetype']
            textoutput += "IOVCheck offline\n"
            textoutput += "DuplicateTagHLT\n"
            textoutput += "DuplicateTagEXPRESS\n"
            textoutput += "DuplicateTagPROMPT %s\n" % aFile['DuplicateTagPROMPT']

            fout = open(filenameTXT, "w")
            fout.write(textoutput)
            fout.close()

            os.chmod(filenameDB, stat.S_IREAD | stat.S_IWRITE | stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IROTH | stat.S_IWOTH)
            os.chmod(filenameTXT, stat.S_IREAD | stat.S_IWRITE | stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IROTH | stat.S_IWOTH)

            if dropboxValidation == "False":

                fout = tarfile.open(filenameTAR, "w:bz2")
                fout.add(filenameDB)
                fout.add(filenameTXT)
                fout.close()

                os.chmod(filenameTAR, stat.S_IREAD | stat.S_IWRITE | stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IROTH | stat.S_IWOTH)

                files2upload.append(filenameTAR)
                files2delete.append(filenameTAR)

            else:

                files2upload.append(filenameDB)
                files2upload.append(filenameTXT)

            files2delete.append(filenameTXT)
            files2delete.append(filenameDB)

        #
        # Hack, still copy out sqlite file
        #
        if targetDir != "":

            if len(files2copy) > 1:
                msg = "AlcaHarvest job wrote more than one sqlite file, this is not supported.\n"
                self.failJob(msg)
                return
            else:
                for file2copy in files2copy:

                    jobSpec = os.environ.get("PRODAGENT_JOBSPEC")
                    fileName = jobSpec.split('/')[-1]
                    fileName = fileName[:-len('-JobSpec.xml')] + ".db"

                    print '==> stage out %s to %s' % (sqlitefile, os.path.join(targetDir, fileName))

                    if 0 != subprocess.call(["rfcp", sqlitefile, os.path.join(targetDir, fileName)]):
                        msg = "stage out of sqlite file failed\n"
                        self.failJob(msg)
                        return

        if dropboxHost == "":

            print "DEBUG no DropBox host specified, skip upload.\n"

        else:

            if dropboxValidation == "False":

                for file2upload in files2upload:

                    status = False
                    if 0 == subprocess.call(["scp", "-p", file2upload, "%s:/tmp/" % dropboxHost]):
                        if 0 == subprocess.call(["ssh", dropboxHost, "mv /tmp/%s /DropBox/" % file2upload]):
                            status = True

                    if status == False:
                        msg = "DropBox upload failed for %s.\n" % file2upload
                        msg += "==> Unable to continue, upload failed or partially complete/incomplete\n"
                        self.failJob(msg)
                        return

            else:

                for file2upload in files2upload:

                    status = False

                    if 0 == subprocess.call(["scp", "-p", file2upload, "%s:/tmp/" % dropboxHost]):
                        if 0 == subprocess.call(["ssh", dropboxHost, "mv /tmp/%s /DropBox_test/" % file2upload]):
                            print "DEBUG validation upload finished"
                            status = True

                    if status == False:
                        msg = "DropBox validation upload failed for %s.\n" % file2upload
                        msg += "==> Unable to continue, upload failed or partially complete/incomplete\n"
                        self.failJob(msg)
                        return

        for file2delete in files2delete:
            os.remove(file2delete)

        return


    def failJob(self, msg):
        """
        _failJob_

        fail job by modifying the job report

        """
        jobRep = self.state._JobReport

        print msg
        error = jobRep.addError(60311, "DQMUploadError")
        error['Description'] = msg
        jobRep.status = "Failed"
        jobRep.exitCode = 60311
        self.state.saveJobReport()
        updateReport(self.toplevelReport, jobRep)
        return


def main():
    """
    _main_

    """
    print "==================AlcaHarvest Post Processing======================"

    harvester = OfflineAlcaHarvester()
    harvester.processJobReport()

    print "==================AlcaHarvest Post Processing Done================="

if __name__ == '__main__':
    main()
