
__revision__ = "$Id: DBSUploadAPI.py,v 1.33 2010/08/17 15:28:10 hufnagel Exp $"
__version__ = "$Revision: 1.33 $"

import logging
import time

from DBSAPI.dbsApi import DbsApi
from DBSAPI.dbsException import *
from DBSAPI.dbsApiException import *

from DBSAPI.dbsPrimaryDataset import DbsPrimaryDataset
from DBSAPI.dbsAlgorithm import DbsAlgorithm
from DBSAPI.dbsRun import DbsRun
from DBSAPI.dbsFileBlock import DbsFileBlock
from DBSAPI.dbsLumiSection import DbsLumiSection
from DBSAPI.dbsFile import DbsFile
from DBSAPI.dbsQueryableParameterSet import DbsQueryableParameterSet
from DBSAPI.dbsProcessedDataset import DbsProcessedDataset

from T0 import Globals
from T0.ExportSystemAPI.ExportSystemExceptions import DBSServiceError

class _DBSObjInfo(dict):
    """
    __DBSObjInfo__
    internally used class holding different dbs objects
    """
    def __init__(self,  **args):
        dict.__init__(self)

        self.setdefault("DBSBLOCK", None)
        self.setdefault("DBSRUN", None)
        self.setdefault("PRIMARY_DATASET", None)
        self.setdefault("ALGORITHM", None)
        self.setdefault("PROCESSED_DATASET", None)
        self.setdefault("TIER_LIST", [])
        self.update(args)

def _insertDBSBlockWithDependency(dbsApi, dbsObjInfo, seList):
    """
    _insertDBSBlockWithDependency
    Insert DBSObjects with dependency order.
    The order has to be preserved according to the DBSObject dependencies
    """
    dbsApi.insertPrimaryDataset(dbsObjInfo["PRIMARY_DATASET"])
    dbsApi.insertAlgorithm(dbsObjInfo["ALGORITHM"])
    dbsApi.insertRun(dbsObjInfo["DBSRUN"])
    dbsApi.insertProcessedDataset(dbsObjInfo["PROCESSED_DATASET"])
    dbsApi.insertBlock(dbsObjInfo["PROCESSED_DATASET"], 
                       block = dbsObjInfo["DBSBLOCK"], 
                       storage_element_list = seList)
    
    
def insertDBSBlocks(dbsApi, blockSet, seList, expressFlag):
    
    # do it in the right order ("other" is a catch all in case some
    # get added we don't already have)
    
    for dataTier in Globals.GlobalValues.TIER_ORDER:
        _insertDBSBlocksByTier(dbsApi, blockSet, seList, dataTier, expressFlag)
    
    _insertDBSBlocksByTier(dbsApi, blockSet, seList, "other", expressFlag)

    return 

def _insertDBSBlocksByTier(dbsApi, blockSet, seList, tier, expressFlag):
    """
    _insertDBSBlocksByTier_
    
    blockSet
    update DBS with block information and update PhEDEx to export file to 
    specified S.E. (storage element) T1, this case.
    
    call DBS API
       1. Create a Primary Dataset (If one doesn't exist already)
       2. Create an Algorithm (Combination of Application and ParametersetID)
       3. Insert Run(s)
       4. Create a Processed Dataset for the Primary dataset, providing data tier etc)
       5. Insert Lumi Section (s)
       6. Create a Block
       7. Insert files into Processed dataset

    """
    
    for blockDict in blockSet.values():
        blockTier = blockDict["block"].getDataTier()
         # Only process the Tier we want or if we don't recognize the
         # tier of the file do it in the catch all "other" iteration
        if blockTier != tier and \
               not ( Globals.GlobalValues.TIER_ORDER.count( blockTier ) == 0 
                     and tier == "other" ):
            continue
        
        fileList = blockDict['fileList']
        if len( fileList ) == 0:
            msg = """ Block %s has empty fileList. This block is closed by run 
                      (or timeout in express case), 
                     but still needed put DBS to mark block is closed 
                     """ % blockDict
            logging.warning(msg)
        else: 
            
            dbsObjInfo = getDBSObjInfo(blockDict['block'])
        
            if blockDict["isNewBlock"]:
                _insertDBSBlockWithDependency(dbsApi, dbsObjInfo, seList)
            try:    
                insertFilesToDBS(dbsApi, blockDict['fileList'],
                                dbsObjInfo, expressFlag)
            except Exception, ex:
                logging.error("Some Exception: %s" % str(ex))
                logging.error("retry inserting to DBS")
                
                try:
                    _insertDBSBlockWithDependency(dbsApi, dbsObjInfo, seList)
                    insertFilesToDBS(dbsApi, blockDict['fileList'],
                                     dbsObjInfo,expressFlag)
                    logging.debug("Retry Successful for files %s" % blockDict['fileList'])
                except Exception, ex:
                    
                    errorMsg = "file List %s\n\n" % blockDict['fileList']
                    errorMsg += "dbs object List %s\n\n" % dbsObjInfo
                    errorMsg += str(ex)
                    crashMessage = Globals.stackTraceLog(errorMsg)
                    # make component crash try to insert files in closed block
                    if str(ex).find("Block Closed") != -1:
                        msg = """Closed block Exception: DBS Block is already closed\n\t%s
                                 Reopening the block, But investigate""" % blockDict['block']
                        dbsApi.openBlock(dbsObjInfo['DBSBLOCK'])
                        raise DBSServiceError(msg)
                    else:
                        raise Exception, crashMessage
                    
        
        # empty blocks are still needed to be processed for status update
        # TODO: this assumes closeBlock DBS call always succeed, 
        # Needs one more status for Block DBSUloaded 
        if blockDict['block']["STATUS"] == "Closed":
            dbsApi.closeBlock(blockDict['block']["BLOCK_NAME"])
            logging.debug("Block ID: %s Name: %s"  %(
                         blockDict['block']["BLOCK_ID"],
                         blockDict['block']["BLOCK_NAME"]))
        
    return
        
            
def getDBSObjInfo(blockInfo):
    
    # check to see whether the run info is already injected.
    #  By retrieving run from DBS.
    # inject runinfo
    # check primary dataset by ret
    
    runConfig = blockInfo.getRunConfigForBlock()
    
    # setup the default value. These will be overwritten depending 
    # on the condition
    globalTag = ""
    psetHash = "NO_PSET_HASH"
    configContent = "PSET_CONTENT_NOT_AVAILABLE"
    configURL = None
    
    if runConfig.isDatasetPathExpress(blockInfo["DATASET_PATH_ID"]):
        appFamily = "Express"
        streamID = runConfig.getStreamIDfromDatasetPathID(blockInfo["DATASET_PATH_ID"])
        expressConfig = runConfig.getExpressConfiguration(streamID)
        configURL = expressConfig.ProcessingConfigURL
        appVersion = expressConfig.CMSSWVersion
        
    elif blockInfo.getDataTier() == "RAW":
        appFamily = "Repacker"
        streamID = runConfig.getStreamIDfromDatasetPathID(blockInfo["DATASET_PATH_ID"])
        repackConfig = runConfig.getRepackConfiguration(streamID)
        appVersion = repackConfig.CMSSWVersion
    
    elif blockInfo.getDataTier() == "RECO" \
         or blockInfo.getDataTier() == "AOD" \
         or blockInfo.getDataTier() == "DQM":
        appFamily = "PromptReco"
        
        recoConfig = runConfig.getRecoConfiguration(blockInfo["DATASET_PATH_ID"])

        # get psetHash from t0ast
        globalTag = recoConfig.GlobalTag
        configURL = recoConfig.ConfigURL
        appVersion = recoConfig.CMSSWVersion
    
    elif blockInfo.getDataTier() == "ALCARECO" \
             or blockInfo.getDataTier() == "ALCAPROMPT":
        appFamily = "AlcaSkim"
        
        recoConfig = runConfig.getRecoConfiguration(blockInfo["DATASET_PATH_ID"])
        alcaConfig = runConfig.getAlcaConfiguration(blockInfo["DATASET_PATH_ID"])
        globalTag = recoConfig.GlobalTag
        configURL = alcaConfig.ConfigURL
        appVersion = alcaConfig.CMSSWVersion
    
    else:
        raise RuntimeError, "Unknown data tier: %s" % blockInfo.getDataTier()
    
    
    parentDataset = blockInfo.getParentDatasetPath()         
    
    if parentDataset == None:
        parentDatasetList = []
    else:
        parentDatasetList = [ parentDataset ]
    if globalTag == None:
        globalTag = ""
        
    primaryDatasetName = blockInfo.getPrimaryDatasetName()
    # need get info from some where Conf?
    logging.debug("BlockName : %s" % blockInfo["BLOCK_NAME"])
    logging.debug("PrimaryDatasetName : %s" % primaryDatasetName)
    processedDatasetName = blockInfo.getProcessedDatasetName()
    logging.debug("processedDatasetName : %s" % processedDatasetName)

    logging.debug("ParentList for %s = %s" % (
                        processedDatasetName,parentDatasetList))
    
    primary = DbsPrimaryDataset (Name = primaryDatasetName, Type="data")  

    # should only update the content if algorithm is not exist
    #if configURL != None:
    #    configPath = runConfig.retrieveConfigFromURL(configURL)
    #    fileHandle = open(configPath, 'r')
    #    configContent = '\n'.join(fileHandle.readLines())
    #    fileHandle.close()
    
    algo = DbsAlgorithm ( ExecutableName ="cmsRun",
                          ApplicationFamily = appFamily,
                          ApplicationVersion = appVersion,
                          # To do :need to get this info from some Config
                          ParameterSetID=DbsQueryableParameterSet(
                              Hash= psetHash,
                          #    Name="MyFirstParam01", 
                          #    Version="V001",
                          #    Type="test",
                          #    Annotation="This is test",
                              Content= configContent
                           )
                         )
    
    logging.debug("run inserted to dbs : %s" % blockInfo["RUN_ID"])
    run = DbsRun ( RunNumber=blockInfo["RUN_ID"],
                   # add fake value for since the info is not available
                   NumberOfEvents= 0, 
                   # add fake value for since the info is not available
                   NumberOfLumiSections= 0,
                   # add fake value for since the info is not available
                   TotalLuminosity = 0,
                   # what is this?
                   StoreNumber= 0,
                   StartOfRun= runConfig.getStartTime(),
                   # add fake value for since the info is not available
                   EndOfRun= 0
                  )
    
    procDataset = \
            DbsProcessedDataset( PrimaryDataset=primary,
                                 Name = processedDatasetName, 
                                 PhysicsGroup="DPG",
                                 Status="Valid",
                                 AcquisitionEra= runConfig.getAcquisitionEra(),
                                 GlobalTag=globalTag,
                                 TierList=[blockInfo.getDataTier()],
                                 AlgoList=[algo],
                                 # what is this primary dataset info?
                                 RunsList=[blockInfo["RUN_ID"]],
                                 ParentList=parentDatasetList
                                )

    dbsBlock = DbsFileBlock ( Name = blockInfo["BLOCK_NAME"] )

    dbsObjInfo = _DBSObjInfo()
    dbsObjInfo['PRIMARY_DATASET'] = primary
    dbsObjInfo['DBSRUN'] = run
    dbsObjInfo['DBSBLOCK'] = dbsBlock
    dbsObjInfo['ALGORITHM'] = algo
    dbsObjInfo['PROCESSED_DATASET'] = procDataset
    dbsObjInfo['TIER_LIST'] = [blockInfo.getDataTier()]

    return dbsObjInfo

    
def insertFilesToDBS(dbsApi, t0astFileList, dbsObjInfo, expressFlag):
    """
    _insertFilesToDBS_
    dbsApi : DbsApi object
    t0astFileList : List of T0ASTFile objects
    dbsBlock : DbsFileBlock object for t0astFileList
    
    (runConf and recoConf is used to get processed dataset info
     : may not need this one)
    
    This function will find lumisection for the file and insert to DBS.
    The duplicate lumisection will be removed
    """
    
    dbsFileList = []
    for t0astFile in t0astFileList:
        # Duplicate lumi will be handled by DBS
        # could filter it here if necessary
        dbsLumiList = []
        
        for lumi in t0astFile.getLumiList():
            lumiSection = \
                 DbsLumiSection ( LumiSectionNumber= lumi,
                                  RunNumber = t0astFile.getRunID(),
                                  # need to get lum start time some where
                                  # lumisection table?
                                  LumiStartTime = 0,
                                  # not yet in the database use 0 for now.
                                  # might need to be calculated and put in T0AST
                                  LumiEndTime = 0, #lumi["END_TIME"],
                                  StartEventNumber = 0, # lumi["StartEventNumber"],
                                  EndEventNumber = 0 #lumi["EndEventNumber"]
                                 )
            #doesn't have to insert lumi section explicitly dbsInsertFile will do that.
            dbsLumiList.append(lumiSection)
        
        if expressFlag:
            #TODO: temporary hack:need to create right parentage
            parentList = []
        else:
            parentList = t0astFile.getParentList()
            
        dbsFile = DbsFile ( LogicalFileName = t0astFile["LFN"],
                            #QueryableMetadata= 'This is a test file',
                            NumberOfEvents = t0astFile["EVENTS"],
                            FileSize = t0astFile["FILESIZE"],
                            Status = 'VALID',
                            ValidationStatus = 'VALID',
                            #file type is 'ROOT' or 'EDM'
                            FileType= 'EDM',
                            Dataset = dbsObjInfo["PROCESSED_DATASET"],
                            #Block= isDictType,
                            AlgoList = [dbsObjInfo["ALGORITHM"]],
                            LumiList= dbsLumiList,
                            TierList= dbsObjInfo['TIER_LIST'],
                            #T0DO: express stream are needed to be handled differenty
                            ParentList = parentList 
                           )
        
        #Handle checksums
        # DBS supports only 3 types of checksum (cksum, adler32, md5)
        for type, value in t0astFile["CHECKSUMS"].items():
            #logging.debug("checksumtest file id: %s" % t0astFile['ID'])
            #logging.debug("checksumtest %s: %s" % (type, value))
            if value in (None, ''):
                continue
            if type.lower() == 'cksum':
                dbsFile['Checksum'] = value
            elif type.lower() == 'md5':
                dbsFile['MD5'] = value
            else:
                # assume all dbs type has first letter uppercase
                dbsFile[type.lower().title()] = value
            
        dbsFileList.append(dbsFile)
    
    #logging.debug("------------ processed dataset : %s" % dbsObjInfo["PROCESSED_DATASET"])
    #logging.debug("------------ file List : %s" % dbsFileList)
    #logging.debug("------------ dbsObjInfo : %s" % dbsObjInfo["DBSBLOCK"])
        
    dbsApi.insertFiles(dbsObjInfo["PROCESSED_DATASET"], 
                       dbsFileList, dbsObjInfo["DBSBLOCK"]) 


# change to pass run object if it gets all other information
def updateDBSRunStatus(dbsApi, datasetPaths, runID, totalEvents=0, totalLumis=0):
    """
    _updateDBSRunStatus_
    
    run object examples:

         run = DbsRun (
                 RunNumber=1,
                 NumberOfEvents= 100,
                 NumberOfLumiSections= 20,
                 TotalLuminosity= 2222,
                 EndOfRun= 'never',
         )
    To do: check whether end of time should record at the same time as
    when processing is done 
    """
#    run = DbsRun ( RunNumber = runID,
#                   # add fake value for since the info is not available
#                   NumberOfEvents = totalEvents, 
#                   NumberOfLumiSections = totalLumis,
#                   TotalLuminosity = 0,
#                   # what is this?
#                   StoreNumber = 0,
#                   EndOfRun = long(time.time()),
#                   )
#    dbsApi.updateRun(run)
    
    for datasetPath in datasetPaths:
        try:
            dbsApi.markPDRunDone(datasetPath,runID)
        except Exception, e:    
            if str(e).find("Already closed") != -1:
                logging.warning(str(e))
            else:
                raise
