"""

"""
import os
import logging

from urllib2 import urlopen, HTTPError, URLError
from urllib import urlencode
import httplib
import urlparse

from T0 import Globals
from T0.ExportSystemAPI.Tier0XMLParser import Tier0XMLParser 
from T0.ExportSystemAPI.PhEDExXMLDrop import makePhEDExDataXML
from T0.ExportSystemAPI.PhEDExXMLDrop import makePhEDExXMLForDatasets
from T0.ExportSystemAPI.ExportSystemExceptions import PhEDExServiceError

def isBlockTransfered(phedexDSUrl, blockInfo, nodeList):
    """
    a temporary stub for testing. 
    It will use getNodeList
    """
    # no subscription has made
    if len(nodeList) == 0:
        return False
    
    #it should just return one value or empty list
    resultList = getNodeList(phedexDSUrl, blockInfo).values()
    # if resultList is empty return False
    if not resultList:
        return False
    else :
        # the first value (list) should be the nodes list for given block 
        replicaNodes = resultList[0]
        
    #logging.debug("replicaNodes %s " % replicaNodes)

    # get custodial flag from run Config?
    
    for node in nodeList:
        if not _contains(replicaNodes, node):
            # if one of the transfers didn't occur, 
            # return False 
            return False
    # if all the node in SubscriptionPolicy has a replica    
    return True

def _contains(list, node):
    """
    return True if list contains node
    otherwise return False
    """
    for item in list:
        if node == item:
            return True
    return False

def getNodeList(phedexDSURL, blockList):
    """
    _getStorageElementList_
    
    This function will return the list of the Nodes
    """
    if type(blockList) == list:
        blockList = blockList 
    else:
        blockList = [blockList]

    lfnList = []
    for blockInfo in blockList:
        lfn = blockInfo["BLOCK_NAME"]
        lfnList.append(('block', lfn)) 
        
    urlbase = phedexDSURL.strip('/') + '/blockReplicas'
    
    #just use new variable for clarity
    arglist = []
    arglist.extend(lfnList)
    # flags that could be added: incomplete, updated_since, created_since
    arglist.append(('complete', 'y'))
    nodeList = []
    try:  
        # Build the xml query to use
        url = urlopen(urlbase, urlencode(arglist))
        nodeList = Tier0XMLParser.getNodeListByBlockName(url)
    except URLError, e:
        msg = "Error retrieving locations Contact PhEDExService: %s," % str(e)
        raise PhEDExServiceError(msg)
    else:
        # Check if the list was empty
        if(not nodeList):
            msg = "No existing fileblock matching %s" % (str(lfnList))
            logging.warning(msg)
    
        # Return what we got
        return nodeList

def subscribeToPhEDEx(phedexDSURL, subscription, dbsUrl):
    urlbase = phedexDSURL.strip('/') + '/subscribe'
    args = []
    
    for node in subscription.nodes:
        args.append(('node', '%s' % node))
        
    logging.debug("phedexsubscription datasetPath :%s" % subscription.getDatasetPaths())    
    xml = makePhEDExXMLForDatasets(dbsUrl, subscription.getDatasetPaths())
    
    args.append(('data', xml))
    args.append(('level', '%s' % subscription.level))
    args.append(('priority', '%s' % subscription.priority))
    args.append(('move', '%s' % subscription.move))
    args.append(('static', '%s' % subscription.static))
    args.append(('custodial', '%s' % subscription.custodial))
    args.append(('group', '%s' % subscription.group))
    args.append(('request_only', '%s' % subscription.request_only))

    try:  
    # Build the xml query to use
        xmlDoc = getPhEDExXMLResponse(urlbase, urlencode(args))
        requestID = Tier0XMLParser.getSubscriptionResponse(xmlDoc)
    
    except URLError, e:
        logging.error("###############################################")
        logging.error("PhEDEx Subscription Error Message:/n%s" % xmlDoc)
        msg = Globals.stackTraceLog(str(e))
        raise PhEDExServiceError(msg)

    else:
        if requestID.has_key("Error"):
            if requestID["Error"] == "Error when making call 'subscribe':  sending request creation email failed":
                # try again without sending emails
                args.append(('no_mail', 'y'))
                try:  
                    # Build the xml query to use
                    xmlDoc = getPhEDExXMLResponse(urlbase, urlencode(args))
                    requestID = Tier0XMLParser.getSubscriptionResponse(xmlDoc)
    
                except URLError, e:
                    logging.error("###############################################")
                    logging.error("PhEDEx Subscription Error Message:/n%s" % xmlDoc)
                    msg = Globals.stackTraceLog(str(e))
                    raise PhEDExServiceError(msg)

                else:
                    if requestID.has_key("Error"):
                        msg = "PhEDEx Subscription Request Failed (after retry without email): %s" % requestID
                        raise RuntimeError, msg
                    else:
                        logging.warning( "Subscription made without email: %s" % args )
                        subscription.setRequesterID(int(requestID["RequesterID"]))
                        logging.info("Subscription successful: %s" % xmlDoc)
                        logging.info(" %s" % requestID)
            else:
                msg = "PhEDEx Subscription Request Failed: %s" % requestID
                raise RuntimeError, msg
        else:
            subscription.setRequesterID(int(requestID["RequesterID"]))
            logging.info("Subscription successful: %s" % xmlDoc)
            logging.info(" %s" % requestID)
        return
      

def injectBlocksToPhEDEx(phedexDSURL, globalDbsUrl, datasetPathSet, node):
    
    """
    _injectBlocksToPhedex_

    This is Tier0 specific function for creating XML file from local dbs
    but fake it with global dbs before injecting to PhEDEx
    
    This function is used as hack for simplifying dbs deleting from PhEDEx

    """
    
    urlbase = phedexDSURL.strip('/') + '/inject'
    args = []
    
    args.append(('node', '%s' % node))
    
    logging.debug("Injecting data to the Node %s" % node)
    
    xml = makePhEDExDataXML(datasetPathSet, globalDbsUrl)
    
    args.append(('data', xml))
    args.append(('verbose', '0'))
    args.append(('strict', '0'))
     
    try:  
    # Build the xml query to use
        xmlDoc = getPhEDExXMLResponse(urlbase, urlencode(args))
        #url = urlopen(urlbase, urlencode(args))
        logging.debug("PhEDExResponse: \n%s" % xmlDoc)
        injectionStat = Tier0XMLParser.getInjectionResponse(xmlDoc)
    
    except URLError, e:
        #TODO: remove logging when start to use Alert system
        logging.error("###############################################")
        logging.error("PhEDEx Inject Error Message:/n%s" % xmlDoc )
        msg = Globals.stackTraceLog(str(e))
        raise PhEDExServiceError(msg)
    
    else:
        if injectionStat.has_key("Error"):
            index1 = injectionStat["Error"].find("file")
            index2 = injectionStat["Error"].find("exists")
            if index1 != -1 and index2 != -1:
                logging.warning("File already exists %s" 
                                % injectionStat["Error"])
            elif (injectionStat["Error"].find("cannot initialise security module") != -1
                  and injectionStat["Error"].find("Could not connect to DB") != -1):
                logging.warning("PhEDEx DB connection error  %s" 
                                % injectionStat["Error"])
            else:
                raise RuntimeError, injectionStat
        else:    
            logging.info("Inject successful: %s" % xmlDoc)
            logging.info("Injection Stat: %s" % injectionStat)

        return


def getKeyCert():
    """
       Gets the User Proxy if it exists otherwise throws an exception
       This code is bollowed from DBSAPI/dbsHttpService.py
    """

    # First presendence to HOST Certificate, This is how it set in Tier0
    if os.environ.has_key('X509_HOST_CERT'):
        cert = os.environ['X509_HOST_CERT']
        key = os.environ['X509_HOST_KEY']
        
    # Second preference to User Proxy, very common
    elif os.environ.has_key('X509_USER_PROXY'):
        cert = os.environ['X509_USER_PROXY']
        key = cert

    # Third preference to User Cert/Proxy combinition
    elif os.environ.has_key('X509_USER_CERT'):
        cert = os.environ['X509_USER_CERT']
        key = os.environ['X509_USER_KEY']

    # Worst case, look for proxy at default location /tmp/x509up_u$uid
    else :
        uid = os.getuid()
        cert = '/tmp/x509up_u'+str(uid)
        key = cert

    #Set but not found
    if not os.path.exists(cert) or not os.path.exists(key):
        msg = "Certificate is not found"
        raise RuntimeError, msg
    
    # All looks OK, still doesn't gurantee proxy's validity etc.
    return key, cert

def getPhEDExXMLResponse(urlbase, params):
    """
    request a POST type request given by urlbase and params 
    """
    
    key, cert = getKeyCert()
    # urlparse changed in Python 3 to urllib.parse use 2to3 to convert when 
    #Python 3 is available
    parsedUrl = urlparse.urlparse(urlbase)
    
    
    # works in python 2.5 or newer
    #if parsedUrl.scheme != 'https' and parsedUrl.scheme != 'HTTPS':
    if parsedUrl[0] != 'https' and parsedUrl[0] != 'HTTPS':
        raise RuntimeError("url scheme is not https : %s scheme: %s" % 
                                 (urlbase, parsedUrl[0]))
    
    # works in python 2.5 or newer
    #conn = httplib.HTTPSConnection(parsedUrl.netloc, key_file=key, cert_file=cert)
    #conn.request('POST', parsedUrl.path, params)
    conn = httplib.HTTPSConnection(parsedUrl[1], key_file=key, cert_file=cert)
    headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}
    
    conn.request('POST', parsedUrl[2], params, headers)
    
    response = conn.getresponse()
    
    if response.status != 200:
        raise PhEDExServiceError("HTTP error: %s :%s" % 
                                 (response.status, response.reason))
    
    return response.read()
