#!/usr/bin/env python

__revision__ = "$"
__version__ = "$"

"""
This parser class is modified version based on the dlsXmlParser in DlS 

"""
from xml.sax import ContentHandler, parseString, make_parser

class ContentHandlerInterface(ContentHandler):
    
    def getResult(self):
        
        fname = "Tier0XMLParser.ContentHadlerInterface.getResult"
        raise NotImplementedError, fname
    
class BlockNodeHandler(ContentHandlerInterface):

    def __init__(self):
        self.mapping = {}
        self.error = False
        
    def startElement(self, name, attributes):
        
        if name == "block":
            self.nodeList = []
            self.blockName = attributes["name"]
            #self.blockName.encode("ascii", "ignore")
            #print self.blockName
                
        elif name == "replica":
            self.nodeName = attributes["node"]
            #self.nodeName.encode("ascii", "ignore")
            if self.nodeName and (self.nodeName not in self.nodeList):
                self.nodeList.append(self.nodeName)
                #print self.nodeList
                
    def endElement(self, name):
        if name == "block":
            self.mapping[self.blockName] = self.nodeList
    
    def getResult(self):
        #print self.mapping
        return self.mapping
    

class InjectResponseHandler(ContentHandlerInterface):

    def __init__(self):
        self.mapping = {}
        self.error = False
 
    def startElement(self, name, attributes):
        
        if name == "stats":
            self.mapping["NewDatasets"] = attributes["new_datasets"]
            self.mapping["NewBlocks"] = attributes["new_blocks"]
            self.mapping["ClosedBlocks"] = attributes["closed_blocks"]
            self.mapping["ClosedDatasets"] = attributes["closed_datasets"]
        
        if name == "error":
            self.error = True
    
    def characters(self, ch):
        if self.error:
            self.mapping["Error"] = ch 
            
    def getResult(self):
        #print self.mapping
        return self.mapping
    

class SubscriptionResponseHandler(ContentHandlerInterface):

    def __init__(self):
        self.mapping = {}
        self.error = False
 
    def startElement(self, name, attributes):
        
        if name == "request_created":
            self.mapping["RequesterID"] = attributes["id"]
            
        if name == "error":
            self.error = True
    
    def characters(self, ch):
        if self.error:
            self.mapping["Error"] = ch 
            
    def getResult(self):
        #print self.mapping
        return self.mapping
        
class Tier0XMLParser(object):
    
    @staticmethod
    def _getParseResult(xmlDoc, handler):
        
        """
        __getParseResult_
        private static method only should be used by other method in the class
        """
        parseString(xmlDoc, handler)
        
        return handler.getResult()
    
    @staticmethod
    def _getParseResultFromUrl(xmlSource, handler):
        
        """
        __getParseResult_
        private static method only should be used by other method in the class
        """
        parser = make_parser()
        parser.setContentHandler(handler)
        parser.parse(xmlSource)
        
        return handler.getResult()
    
    @staticmethod
    def getNodeListByBlockName(url):
        """
        Returns a list of dict holding the FileBlock name and list of location
        information contained in the specified XML source (in PhEDEx's 
        blockReplicas format)
        
        @param url: url for response
        
        @return: a list of dict. {block_name: [se1, se2, ..]}
        """
        
        return Tier0XMLParser._getParseResultFromUrl(url, 
                                              BlockNodeHandler())
    
    @staticmethod
    def getInjectionResponse(xmlDoc):
        """
        Returns a list of dict holding the FileBlock name and list of location
        information contained in the specified XML source (in PhEDEx's 
        blockReplicas format)
        
        @param xmlDoc: XML document
        
        @return: a stat info when it is successful, or error message
        """
        
        return Tier0XMLParser._getParseResult(xmlDoc, 
                                                 InjectResponseHandler())
        
    @staticmethod
    def getSubscriptionResponse(xmlDoc):
        """
        Returns a list of dict holding the FileBlock name and list of location
        information contained in the specified XML source (in PhEDEx's 
        blockReplicas format)
        
        @param xmlDoc: XML document
        
        @return: a requester id when it is successful, or error message
        """
        
        return Tier0XMLParser._getParseResult(xmlDoc, 
                                                 SubscriptionResponseHandler())