#!/usr/bin/env python
"""
_MergeJob_

JobSpec Factory for building Merge jobspecs
"""

__revision__ = "$Id: MergeJob.py,v 1.30 2011/01/06 00:01:32 hufnagel Exp $"
__version__ = "$Revision: 1.30 $"

import os
import logging

from T0.JobFactory.FactoryInterface import FactoryInterface

class MergeJob(FactoryInterface):
    """
    _MergeJob_

    JobSpec Factory for building Merge jobspecs

    """
    def __init__(self, workflowSpec):
        FactoryInterface.__init__(self, workflowSpec)

        self.run = self.workflow.parameters["RunNumber"]
        self.cmssw = {}
        self.cmssw["Version"] = self.workflow.parameters["CMSSWVersion"]
        self.cmssw["ScramArch"] = self.workflow.parameters["ScramArch"]
        self.cmssw["CMSPath"] = self.workflow.parameters["CMSPath"]

    def __call__(self, jobID, datatier, inputFiles, jobSpecBaseDir):
        """
        _operator()_

        Create a Merge jobspec

        """
        jobName = "Merge%s-Run%s-%s" % (datatier, self.run, jobID)

        jobSpec = self.workflow.createJobSpec()

        jobSpecDir = os.path.join(jobSpecBaseDir,
                                  str((jobID / 1000) % 1000).zfill(4))
        if not os.path.isdir(jobSpecDir):
            os.makedirs(jobSpecDir)

        jobSpecFileName = jobName + "-jobspec.xml"
        jobSpecFile = os.path.join(jobSpecDir, jobSpecFileName)

        jobSpec.setJobName(jobName)
        jobSpec.setJobType("Merge")

        jobSpec.parameters["RunNumber"] = self.run
        jobSpec.parameters["MaxEvents"] = -1
        jobSpec.parameters["SkipEvents"] = 0
        jobSpec.parameters['JobSpecFile'] = jobSpecFile

        # used for the PromptSkimming system
        if self.workflow.parameters.has_key("OnlySites"):
            if self.workflow.parameters["OnlySites"] != None:
                if type(self.workflow.parameters["OnlySites"]) == str:
                    jobSpec.siteWhitelist.append(self.workflow.parameters["OnlySites"])
                else:
                    logging.debug("YEAH: %s" % self.workflow.parameters["OnlySites"])
                    jobSpec.siteWhitelist.extend(self.workflow.parameters["OnlySites"])      

        cfgInterface = jobSpec.payload.cfgInterface
        cfgInterface.inputFiles = inputFiles

        # finally, save the file (PA needs this)
        jobSpec.save(jobSpecFile)
        logging.debug("JobSpec file saved as %s" % jobSpecFile)

        return jobSpec
