#!/usr/bin/env python
"""
_MergePackJob_

JobSpec Factory for building ExpressMerge jobspecs
"""

__revision__ = "$Id: MergePackJob.py,v 1.4 2010/07/20 15:54:33 hufnagel Exp $"
__version__ = "$Revision: 1.4 $"

import os
import logging

from T0.JobFactory.FactoryInterface import FactoryInterface

class MergePackJob(FactoryInterface):
    """
    _MergePackJob_

    JobSpec Factory for building ExpressMerge jobspecs
    
    """
    def __init__(self, workflowSpec):
        FactoryInterface.__init__(self, workflowSpec)

        self.run = self.workflow.parameters["RunNumber"]
        self.cmssw = {}
        self.cmssw["Version"] = self.workflow.parameters["CMSSWVersion"]
        self.cmssw["ScramArch"] = self.workflow.parameters["ScramArch"]
        self.cmssw["CMSPath"] = self.workflow.parameters["CMSPath"]

    def __call__(self, jobID, datatier, inputFiles, jobSpecBaseDir):
        """
        _operator()_

        Create an ExpressMerge jobspec
        
        """
        jobName = "MergePack-Run%s-%s" % (self.run, jobID)

        jobSpec = self.workflow.createJobSpec()

        jobSpecDir = os.path.join(jobSpecBaseDir,
                                  str((jobID / 1000) % 1000).zfill(4))
        if not os.path.isdir(jobSpecDir):
            os.makedirs(jobSpecDir)

        jobSpecFileName = jobName + "-jobspec.xml"
        jobSpecFile = os.path.join(jobSpecDir, jobSpecFileName) 

        jobSpec.setJobName(jobName)
        jobSpec.setJobType("Express")

        jobSpec.parameters["RunNumber"] = self.run
        jobSpec.parameters["MaxEvents"] = -1
        jobSpec.parameters["SkipEvents"] = 0
        jobSpec.parameters['JobSpecFile'] = jobSpecFile

        cfgInterface = jobSpec.payload.cfgInterface
        cfgInterface.inputFiles = inputFiles

        # finally, save the file (PA needs this)
        jobSpec.save(jobSpecFile)
        logging.debug("JobSpec file saved as %s" % jobSpecFile)

        return jobSpec
