#!/usr/bin/env python
"""
_PromptCalibJob_

Factory object for generating PromptCalib JobSpecs from
a PromptCalib workflow spec.
"""

__revision__ = "$Id: PromptCalibJob.py,v 1.4 2010/04/23 07:24:26 naderer Exp $"
__version__ = "$Revision: 1.4 $"

import os
import logging

from T0.JobFactory.FactoryInterface import FactoryInterface

class PromptCalibJob(FactoryInterface):
    """
    _PromptCalibJob_

    Factory object for generating PromptCalib JobSpecs from
    a PromptCalib workflow spec.
    """
    def __init__(self, workflowSpec):
        FactoryInterface.__init__(self, workflowSpec)

        self.run = self.workflow.parameters["RunNumber"]
        self.WorkflowName = self.workflow.parameters["WorkflowName"]
        self.cmssw = {}
        self.cmssw["Version"] = self.workflow.parameters["CMSSWVersion"]
        self.cmssw["ScramArch"] = self.workflow.parameters["ScramArch"]
        self.cmssw["CMSPath"] = self.workflow.parameters["CMSPath"]

    def __call__(self, jobID, alcaStream, wmbsFiles, maxEvents, skipEvents,
                 maxLumis, skipLumis, jobSpecBaseDir):
        """
        _operator()_

        Creat a PromptCalib jobspec.
        
        """
        #jobName: PromptCalib<calibTask>-Run<runNumber>-<alcaStream>-<jobId>
        jobName = "%s-%s" % (self.WorkflowName, jobID)
        
        jobSpec = self.workflow.createJobSpec()

        jobSpecDir = os.path.join(jobSpecBaseDir,
                                  str((jobID / 1000) % 1000).zfill(4))
        if not os.path.isdir(jobSpecDir):
            os.makedirs(jobSpecDir)

        jobSpecFileName = jobName + "-jobspec.xml"
        jobSpecFile = os.path.join(jobSpecDir, jobSpecFileName)

        jobSpec.setJobName(jobName)
        jobSpec.setJobType("Processing")
        
        jobSpec.parameters["RunNumber"] = self.run
        jobSpec.parameters["MaxEvents"] = maxEvents
        jobSpec.parameters["SkipEvents"] = skipEvents
        jobSpec.parameters["MaxLumis"] = maxLumis
        jobSpec.parameters["SkipLumis"] = skipLumis
        jobSpec.parameters['JobSpecFile'] = jobSpecFile

        cfgInterface = jobSpec.payload.cfgInterface
        cfgInterface.inputFiles = []

        for wmbsFile in wmbsFiles:
            cfgInterface.inputFiles.append(wmbsFile["lfn"])

        # finally, save the file (PA needs this)
        jobSpec.save(jobSpecFile)
        logging.debug("JobSpec file saved as %s" % jobSpecFile)

        return jobSpec
