#!/usr/bin/env python
"""
_PromptRecoJob_

JobSpec Factory for building PromptReco jobspecs
"""

__revision__ = "$Id: PromptRecoJob.py,v 1.19 2010/05/04 15:20:28 hufnagel Exp $"
__version__ = "$Revision: 1.19 $"

import os
import logging

from T0.JobFactory.FactoryInterface import FactoryInterface

class PromptRecoJob(FactoryInterface):
    """
    _PromptRecoJob_

    JobSpec Factory for building PromptReco jobspecs
    
    """
    def __init__(self, workflowSpec):
        FactoryInterface.__init__(self, workflowSpec)

        self.run = self.workflow.parameters["RunNumber"]
        self.cmssw = {}
        self.cmssw["Version"] = self.workflow.parameters["CMSSWVersion"]
        self.cmssw["ScramArch"] = self.workflow.parameters["ScramArch"]
        self.cmssw["CMSPath"] = self.workflow.parameters["CMSPath"]

    def __call__(self, jobID, primaryDataset, inputFiles, maxEvents, skipEvents,
                 jobSpecBaseDir):
        """
        _operator()_

        Create a PromptReco jobspec

        """
        jobName = "PromptReco-Run%s-%s-%s" % (self.run, primaryDataset, jobID)

        jobSpec = self.workflow.createJobSpec()

        jobSpecDir = os.path.join(jobSpecBaseDir,
                                  str((jobID / 1000) % 1000).zfill(4))
        if not os.path.isdir(jobSpecDir):
            os.makedirs(jobSpecDir)

        jobSpecFileName = jobName + "-jobspec.xml"
        jobSpecFile = os.path.join(jobSpecDir, jobSpecFileName)

        jobSpec.setJobName(jobName)
        jobSpec.setJobType("Processing")

        jobSpec.parameters["RunNumber"] = self.run
        jobSpec.parameters["MaxEvents"] = maxEvents
        jobSpec.parameters["SkipEvents"] = skipEvents
        jobSpec.parameters['JobSpecFile'] = jobSpecFile

        cmsswConfig = jobSpec.payload.cfgInterface
        cmsswConfig.inputFiles = inputFiles

        # finally, save the file (PA needs this)
        jobSpec.save(jobSpecFile)
        logging.debug("JobSpec file saved as %s" % jobSpecFile)

        return jobSpec
