#!/usr/bin/env python
"""
_StreamerJob_

"""

__revision__ = "$Id: StreamerJob.py,v 1.8 2011/01/06 00:03:47 hufnagel Exp $"
__version__ = "$Revision: 1.8 $"

import os
import logging

from T0.JobFactory.FactoryInterface import FactoryInterface

class StreamerJob(FactoryInterface):
    """
    _StreamerJob_

    JobSpec Factory for building Repack or Express job specs

    """
    def __init__(self, workflowSpec):
        FactoryInterface.__init__(self, workflowSpec)

        self.run = self.workflow.parameters['RunNumber']
        self.cmssw = {}
        self.cmssw['Version'] = self.workflow.parameters["CMSSWVersion"]
        self.cmssw['ScramArch'] = self.workflow.parameters['ScramArch']
        self.cmssw['CMSPath'] = self.workflow.parameters['CMSPath']


    def __call__(self, jobType, jobID, inputFiles, jobSpecBaseDir, forceUnmergedOutput = False):
        """
        _operator()_

        Create a Repack jobspec

        """
        jobName = "%s-Run%s-%s" % (jobType, self.run, jobID)

        jobSpec = self.workflow.createJobSpec()

        jobSpecDir = os.path.join(jobSpecBaseDir,
                                  str((jobID / 1000) % 1000).zfill(4))
        if not os.path.isdir(jobSpecDir):
            os.makedirs(jobSpecDir)

        jobSpecFileName = jobName + "-jobspec.xml"
        jobSpecFile = os.path.join(jobSpecDir, jobSpecFileName) 

        jobSpec.setJobName(jobName)
        jobSpec.setJobType(jobType)

        jobSpec.parameters['RunNumber'] = self.run
        jobSpec.parameters['JobSpecFile'] = jobSpecFile

        cfgInterface = jobSpec.payload.cfgInterface
        cfgInterface.inputFiles = inputFiles

        #
        # bit of a hack, remove stageout to merged option
        #
        if forceUnmergedOutput:
            for datasetInfo in jobSpec.payload._OutputDatasets:
                if datasetInfo.has_key("MergedLFNBase"):
                    del datasetInfo["MergedLFNBase"]
            for moduleInstance in cfgInterface.outputModules.values():
                if moduleInstance.has_key("MergedLFNBase"):
                    del moduleInstance["MergedLFNBase"]

        # finally, save the file (PA needs this)
        jobSpec.save(jobSpecFile)
        logging.debug("JobSpec file saved as %s" % jobSpecFile)

        return jobSpec
