#!/usr/bin/env python
"""
_AlcaSkim_

File based splitting algorithm that observes the jobgroups the
input files were created with and recombines them for processing.
"""

__revision__ = "$Id: AlcaSkim.py,v 1.2 2010/04/15 08:21:06 hufnagel Exp $"
__version__  = "$Revision: 1.2 $"

from sets import Set
import threading

from WMCore.WMBS.File import File

from WMCore.JobSplitting.JobFactory import JobFactory
from WMCore.Services.UUID import makeUUID
from WMCore.DAOFactory import DAOFactory

class AlcaSkim(JobFactory):
    """
    Split jobs by set of files

    """
    def algorithm(self, groupInstance = None, jobInstance = None, *args,
                  **kwargs):
        """
        _algorithm_

        A file based splitting algorithm, observing jobgroups
        that created the input files.

        """
        myThread = threading.currentThread()
        daoFactory = DAOFactory(package = "T0.State.Database",
                                logger = myThread.logger,
                                dbinterface = myThread.dbi)

        getFilesDAO = daoFactory(classname = "ListFilesForAlcaSkim")
        fileset = getFilesDAO.execute(self.subscription['id'])
        self.newGroup()

        jobDict = {}
        for f in fileset:

            if not jobDict.has_key(f['file_parent']):
                self.newJob(name = makeUUID())
                self.currentJob['mask'].setMaxAndSkipEvents(-1, 0)
                jobDict[f['file_parent']] = self.currentJob

            loadedFile = File(id = f['file_id'])
            jobDict[f['file_parent']].addFile(loadedFile)

        return
