#!/usr/bin/env python
"""
_PromptReco_

Event based splitting algorithm that will chop a fileset into
a set of jobs based on event counts.  This is meant only to be run
in the T0 and will only return jobs for runs that have been enabled
for RECO.
"""

__revision__ = "$Id: PromptReco.py,v 1.7 2010/07/21 13:23:52 hufnagel Exp $"
__version__  = "$Revision: 1.7 $"

import threading

from WMCore.WMBS.File import File

from WMCore.JobSplitting.JobFactory import JobFactory
from WMCore.Services.UUID import makeUUID
from WMCore.DAOFactory import DAOFactory

class PromptReco(JobFactory):
    """
    Split jobs by number of events

    """
    def algorithm(self, groupInstance = None, jobInstance = None, *args,
                  **kwargs):
        """
        _algorithm_

        An event base splitting algorithm.  All available files are split into a
        set number of events per job.  

        """
        myThread = threading.currentThread()
        daoFactory = DAOFactory(package = "T0.State.Database",
                                logger = myThread.logger,
                                dbinterface = myThread.dbi)
        
        getFilesDAO = daoFactory(classname = "ListFilesForReco")
        fileset = getFilesDAO.execute(self.subscription["id"])

        if len(fileset) > 0:
            self.newGroup()

        for f in fileset:
            loadedFile = File(id = f['file_id'])
            eventsInFile = f['events']

            if eventsInFile == 0:
                self.newJob(name = makeUUID())
                self.currentJob.addFile(loadedFile)
                self.currentJob['mask'].setMaxAndSkipEvents(-1, 0)
            else:
                jobsCreated = 0
                currentEvent = 0
                while currentEvent < eventsInFile:
                    self.newJob(name = makeUUID())
                    self.currentJob.addFile(loadedFile)
                    self.currentJob['mask'].setMaxAndSkipEvents(f['reco_split'], currentEvent)
                    currentEvent += f['reco_split']
                    jobsCreated += 1

        return
