"""
_MotT0 request handler_
 This class sorts out the different MotT0 requests
"""

from WMCore.WebTools.Page import TemplatedPage
from cherrypy import expose
import cherrypy, TableContents, os,Utils,PlotContents


class MotT0(TemplatedPage):
    """
    MotT0 main page
    This page is the request handler: every call (page/static resource/AJAX call) is maneged! 
    """
    PATH = os.path.abspath(os.path.dirname(__file__))
    _cp_config = {'tools.staticdir.on' : True,
                  'tools.staticdir.dir' : PATH+"/static/"
    }
    
    
    def __init__(self, config={}):
      TemplatedPage.__init__(self, config)
      res,log=Utils.configurationCheck(self)
      if not(res):
         raise Exception('Invalid Configuration: '+log)
     
    @expose
    def index(self, **kwargs):
        return self.default(kwargs=kwargs)
    
    
    @expose 
    def prova(self, *args, **kwargs):
      return "OK"
    
    @expose 
    def default(self, *args, **kwargs):
     #args contains the page name to be displaied
     #kwargs contains the parameres
     
     #checking configuration options: if failed shows error

       
     #increase Cherrypy Timeout
     cherrypy.config.update({'tools.expires.secs': 800,'tools.expires.on':False})
     cherrypy.tools.expires.sec= 800
     cherrypy.response.timeout = 800
     #print "CherryPy Timeout:"+str(cherrypy.response.timeout) +"  "+str(cherrypy.config['tools.expires.secs'])
     
     
     
        ###########################################static content request
     if len(args)>0 and args[0]=='static':
       try:
         file=kwargs['file']
         # security checks are made by the getCachedFile function itself
         return Utils.getCachedFile(file) 
       except Exception,e:
          print e
          return None
   
     ############################################################################ Pages handler
     page=None
     if len(args)==0 or args[0]=='system': 
       from MotT0Index import MotT0CachedIndex # DAS may not work but there may be a chace hit.
       page=MotT0CachedIndex(self,kwargs)
     else:
       ###check for DAS availabilty
       if not(Utils.workingDAS(self.config.DAS)):
         Utils.sendAlarm(self.config.emailAlert,self.config.DAS,"Impossible to connect to DAS")
         return Utils.DAS_ERROR_MSG
       ###### other pages
       if args[0]=='detail':
          from MotT0Detail import MotT0Detail
          page=MotT0Detail(self,kwargs)
       if args[0]=='run':
          from MotT0Run import MotT0Run
          page=MotT0Run(self,kwargs)
       if args[0]=='history':
          from MotT0History import MotT0History
          page=MotT0History(self,kwargs)
       if args[0]=='plots':
          from MotT0Plots import MotT0Plots
          page=MotT0Plots(self,kwargs)
       if args[0]=='statplots':
          from MotT0Plots import MotT0StatPlots
          page=MotT0StatPlots(self,kwargs)
       #################### service and development pages
       #if args[0]=='service':
          #page=MotT0Service(self,kwargs)
       #if args[0]=='ajax':
          #return self.templatepage('ajax-test', {} )
        ##add other pages   
        
       
       ################################################## single components, used for Ajax scripts
       if args[0]=='heart':
          from Heartbeats import HeartThread
          t=HeartThread(self.config.DAS)
          t.start()
          t.join()
          return t.getHTML()
       # statistic tables component
       if args[0]=='stats':
          from Statistics import StatThread
          try:
             run=str(int(kwargs['run_id']))
          except:
             run=""
          t=StatThread(self.config.DAS,run)
          t.start()
          t.join()
          return t.getHTML()
       #################### general porpouse requests! AJAX calls!!!
       if args[0]=='single':
         try:
           if 'plot' in kwargs.keys():
             from PlotCode import plotThread
             name=kwargs['plot']
             m=getattr(PlotContents,name)
             content=m(kwargs)
             t=plotThread(self.config.DAS,self.config.plotfairy,content)
           if 'table' in kwargs.keys():
             from Table import tableThread
             name=kwargs['table']
             #print "NAME="+name
             m=getattr(TableContents,name)
             content=m(kwargs)
             t=tableThread(self.config.DAS,content) 
                
           t.start()
           t.join(300)
           res= t.getHTML()
           if res==None: raise Exception('VoidResult')
           return res 
         except  Exception, e:
             print e
             error="<b>Connection Error or Wrong URL</b> <BR/> "
             #try:
               #error+="<BR/> More information: <BR/>"+ content.message
             #except:
               #print "ARGS="+str(args)+"\n" +e
             return error
                 
                 
       #if args[0]=='examplePlots':
           #from plotExamples import *
           #return getPlotExamples()
       #if args[0]=='tmp':
           #params={"tmplPath": self.templatedir+"/"} 
           #return self.templatepage('tmp', **params ) 
           
       # Error page
       if page==None:
          msg="Wrong page request! Check URL."
          eParams={
            "title": "Error",
            "tmplPath": self.templatedir+"/",
            "msg": msg
              }
          return self.templatepage('error', **eParams )
       
     return page.getTemplate()
          
#######################################################################
import time
class MotT0Page(object):
    
    def __init__(self,motT0Template,kwargs={}):
      """ MotT0 page is the superclass for the pages: index, history, detail,run,plots"""
      self.params=kwargs
      self.motT0Template=motT0Template
      self.DASUrl=motT0Template.config.DAS
      self.templatedir = motT0Template.templatedir+'/'
      self.plotfairyUrl=motT0Template.config.plotfairy
    
    def errorPage(self,msg="Empty Page"):
          eParams={
            "title": "Error",
            "time" : time.ctime(),
            "tmplPath": self.templatedir+"/",
            "msg": msg
              }
          return self.motT0Template.templatepage('error', **eParams )
    
    def getTemplate():
         return self.errorPage() 
##########################################################################