"""
_MotT0Plots Page_

Pages with  collection of plots: statistical plots and tier plots
""" 
from WMCore.WebTools.Page import TemplatedPage
from MotT0 import MotT0Page
from HTTPfetcher import *
from  datetime import datetime
from PlotLib import *

"""
_MotT0StatPlots Page_

This page shows 1 plot for every process: (express | repack | reco | alca) + [merge] 

""" 
from PlotContents import *

class MotT0StatPlots(MotT0Page):
    title="Job Lifetime Plots"
    tableTitle="Statistical Tables"
    
    def __init__(self,motT0template,kwargs={}):
      MotT0Page.__init__(self,motT0template,kwargs)
      #dummy plot used to process the parameters
      self.plotContent=statPlotContent(kwargs)

    def getTemplate(self):
      #build  params URL string for AJAX objects
      if self.plotContent.run>0: 
        self.tableTitle+=" for run: <span style=\"color:red\"> %s </span>"%(self.plotContent.run)
      else: self.tableTitle+=" for the <span style=\"color:red\"> whole system </span>" 
      self.params=""
      if self.plotContent.run>0:
        self.params+="run_id=%d&"%(self.plotContent.run)
      if self.plotContent.autotime==False:
        self.params+="hours=%d&"%(self.plotContent.hours)
        self.params+="step=%d&"%(self.plotContent.step)
        d=datetime.fromtimestamp(self.plotContent.endTime)
        self.params+="sh=%d&sd=%d&sm=%d&sy=%d"%(d.hour,d.day,d.month,d.year)
      else:
        d=datetime.fromtimestamp(mktime(localtime()))
        self.plotContent.step=5
        self.params+="autotime=1&"
      #print "PARAMS==="+str(self.params)+ "   MESSAGE=" +str(self.plotContent.message)
      
      #GlobalStat special behavior in the javascript in templates/header.html
      form=dateForm('GlobalStat',d,self.plotContent.hours,self.plotContent.hourRange,self.plotContent.step, self.plotContent.stepRange,self.plotContent.run)
      message= self.plotContent.message
      if len(self.plotContent.message)>0: 
        message="<B>General parameter log:</B>"+message+"<BR/><BR/>"
      params={
            "title": self.title,
            "tableTitle": self.tableTitle,
            "time" : ctime(),
            "tmplPath": self.templatedir,
            "params":self.params,
            "dateForm":message+form
              }
      # if the page is a run-sepcific I add the link to the run detail
      if  self.plotContent.run>0:  
          params.update({'topLinks':"""<a href="run?run_id=%s"><img border="0" src="imgs/table.png" alt="run-tables" title="tables of run %s" width="25"> </a>"""%(self.plotContent.run,self.plotContent.run)})
      return self.motT0Template.templatepage('statplots', **params )



##################################################################3


"""
_MotT0Plots Page_

This page shows: # of events, event sizes and data rate plots for:
        * all the streams
        * A streams
        * Express streams
        * 'other' streams
Small exception for the 'streamers' page: all the info is contained in 3 plots.
""" 
class MotT0Plots(MotT0Page):
    title="Plots"
    tableTitle="Plots for <span style=\"color:red\"> %s TIER</span>"
    
    def __init__(self,motT0template,kwargs={}):
      MotT0Page.__init__(self,motT0template,kwargs)
      #dummy plot used to process the parameters
      self.plotContent=tierPlotContent(kwargs)
      

  
    def getTemplate(self):
      if self.plotContent.tier.lower() not in ['','express','raw','reco','alcareco']:
        return self.errorPage('Tier option not valid!')
      #build  params URL string for AJAX objects
      if self.plotContent.tier=="":
        self.tableTitle=self.tableTitle.replace('TIER','STREAMERS')
      self.params="tier="+self.plotContent.tier+"&"
      if self.plotContent.autotime==False:
        self.params+="hours=%d&"%(self.plotContent.hours)
        self.params+="step=%d&"%(self.plotContent.step)
        d=datetime.fromtimestamp(self.plotContent.endTime)
        self.params+="sh=%d&sd=%d&sm=%d&sy=%d&"%(d.hour,d.day,d.month,d.year)
      else: 
        self.params+="autotime=1&"
        d=datetime.fromtimestamp(mktime(localtime()))
        self.plotContent.step=5
      
      
      self.params+="loadsiblings=1"
      self.tableTitle=self.tableTitle%(self.plotContent.tier)
      #Global special behavior in the javascript in templates/header.html
      form=dateForm('Global',d,self.plotContent.hours,self.plotContent.hourRange,self.plotContent.step, self.plotContent.stepRange,hiddenValues="""<input type="hidden" name="tier" value="%s">"""%(self.plotContent.tier))
      message= self.plotContent.message
      if len(self.plotContent.message)>0: 
        message="<B>General parameter log:</B>"+message+"<BR/><BR/>"
      params={
            "title": self.title,
            "tableTitle": self.tableTitle,
            "time" : ctime(),
            "params":self.params,
            "tmplPath": self.templatedir,
            "tier": str(self.plotContent.tier),
            "dateForm":message+form,
            "fileTime":""
            }
      return self.motT0Template.templatepage('plots', **params )
