#from Table import *
from Columns import *
from types import *

class content(object):
  """
This class represent the SuperClass of every table content information.
Every table may have different title/URL and columns!
The __init__ method is also used to set the page attribute: in long table this decide the limits of the data
The 'name' of the content MUST be the same of the class [__class__.__name__]
  """ 
  page=1
  suffix=""
  colDecorators=[]
  title=""
  linkPage=""
  isLongTable=False
  countQuery=""
  tableStepSize=10
  tableClass='Table'
  message=""
  
  def __init__(self,params={}):
    self.params=params
    self.checkParams()
    self.updateLinks() 
    
  def checkParams(self): 
    self.downLimit =0   
    self.upLimit  = self.tableStepSize
    self.run=0
    try:
       if str(self.params['table']) ==self.name:
         #if more values of ['table'] this is ignored.
         try:
            self.run=int(self.params['run_id'])
         except:
            self.run=0
         try:
           self.page=int(self.params['page'])
           if (self.page<1):self.page=1
           self.downLimit =(self.page -1) * self.tableStepSize 
           self.upLimit=self.downLimit+self.tableStepSize
         except:
           self.downLimit =0 
           self.upLimit  = self.tableStepSize
         try:
            self.status=self.params['status']
         except:
           pass    
    except Exception, e:
       #print str(e)
       pass
    
 
  def updateLinks(self):
      """ This method is overriden in some tables and used to update the connection links according with the parameter values
      """
      pass
  
     


class longTableContent(content):
   """ This kind of table can be split in several 'pages', therefore there is a call for having the total number of row (countURL)
   """
   isLongTable=True
   linkPage=""
   forceRunParameter=False
   
   def updateLinks(self):
       
       if self.run>0:
           self.URL+="?run="+str(self.run)+'&'
           self.countURL+="?run="+str(self.run)+'&'
       elif self.forceRunParameter:
           self.URL+='?run=&'
           self.countURL+='?run='
       else: self.URL+='?'
       self.URL+="uplimit="+str(self.upLimit) +'&downlimit='+str(self.downLimit)
       try:
         self.URL+="&status="+self.status
         self.countURL+="&status="+self.status
         self.title=self.title%(self.status)
       except Exception, e:
         #print str(e)
         pass
       

############################################ main page tables

class runDetails(content):
   """ RunDetails is the main table of the homepage
   """
   name='runDetails'
   URL='lastruns'
   title="Runs: %s"
   linkPage='system?'
   isLongTable=True      
   tableStepSize=10
   page=1
   tableClass="RunDetails"
   colDecorators=[RunId(),sTime(),eTime(),process(),version(),hlt(),
           acq(),status(),numstreamers(),expressGraph(),repackGraph(),recoGraph(),alcaGraph(),plotLinks()]
           
########################################################################################################
class exprJ(longTableContent):
  """ expressJobs table: homepage"""
  name='exprJ'
  #CSSclass='expressBG'
  URL='expressnewusedjobs'
  countURL="expressnewusedjobs_count"
  title="Express New and Used Jobs"
  path='history?source=Express'
  colDecorators=[JobId(path),RunId(),defTime(),nstreamers()]
  isLongTable=True

class exprMJ(longTableContent):
  """ expressMerge Jobs table: homepage"""
  name='exprMJ'
  #CSSclass='expressBG'
  URL='expressmergelastjobs'
  countURL="expressmergelastjobs_count"
  title="Express Merge Acquired Jobs"
  path='history?source=ExpressMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum()]
  isLongTable=True  
    
class repkJ(longTableContent):
  """ repack Jobs table: homepage"""
  name='repkJ'
  #CSSclass='repackBG'
  URL='repacklastacquired'
  countURL="repacklastacquired_count"
  title="Repack running Jobs"
  path='history?source=Repack'
  colDecorators=[JobId(path),RunId(),runTime(attrNames=["def_time"]),nstreamers(),stream()]
  isLongTable=True
  
class repkMJ(longTableContent):
  """ repack Merge Jobs table: homepage"""
  name='repkMJ'
  #CSSclass='repackBG'
  URL='repackmergelastacquired'
  countURL="repackmergelastacquired_count"
  title="Repack Merge Acquired Jobs"
  path='history?source=RepackMerge'
  colDecorators=[JobId(path),RunId(),defTime(attrNames=["state_time"]),filesNum(),dataset()]
  isLongTable=True  
  
class recoJ(longTableContent):
  """ reco Jobs table: homepage"""
  name='recoJ'
  #CSSclass='recoBG'
  URL='recolastacquired'
  countURL="recolastacquired_count"
  title="Reconstruction Acquired Jobs"
  path='history?source=Reco'
  colDecorators=[JobId(path),RunId(),defTime(attrNames=["state_time"]),filesNum(),dataset()]
  isLongTable=True
  
class recoMJ(longTableContent):
  """ reco Merge Jobs table: homepage"""
  name='recoMJ'
  #CSSclass='recoBG'
  URL='recomergelastacquired'
  countURL="recomergelastacquired_count"
  title="Reco Merge Acquired Jobs"
  path='history?source=RecoMerge'
  colDecorators=[JobId(path),RunId(),defTime(attrNames=["state_time"]),filesNum(),dataset()]
  isLongTable=True
    
class alcaJ(longTableContent):
  """ alca Jobs table: homepage"""
  name='alcaJ'
  #CSSclass='alcaBG'
  URL='alcalastacquired'
  countURL="alcalastacquired_count"
  title="Alcaskim Acquired Jobs"
  path='history?source=Alca'
  colDecorators=[JobId(path),RunId(),defTime(attrNames=["state_time"]),filesNum(),dataset()]
  isLongTable=True
  
class alcaMJ(longTableContent):
  """ alca Merge Jobs table: homepage"""
  name='alcaMJ'
  #CSSclass='alcaBG'
  URL='alcamergelastacquired'
  countURL="alcamergelastacquired_count"
  title="AlcaSkim Merge Acquired Jobs"
  path='history?source=AlcaMerge'
  colDecorators=[JobId(path),RunId(),defTime(attrNames=["state_time"]),filesNum(),dataset()]
  isLongTable=True
    
################################################################################ detail tables

class exprJByStatus(exprJ):
  """ Express Jobs by Status table: detail page"""
  name='exprJByStatus'
  URL='expressjobsbystatus'
  countURL='expressjobsbystatus_count'
  title="Express %s Jobs"
  forceRunParameter=True
  tableStepSize=100
  
class exprMJByStatus(exprMJ):
  """ Express Merge Jobs by Status table: detail page"""
  name='exprMJByStatus'
  URL='expressmergejobsbystatus'
  countURL='expressmergejobsbystatus_count'
  title="Express Merge %s Jobs"  
  colDecorators=[JobId(path='history?source=ExpressMerge'),RunId(),defTime(attrNames=["state_time"]),filesNum()]
  forceRunParameter=True
  tableStepSize=100  
  
class repkJByStatus(repkJ):
  """ Repack Jobs by Status table: detail page"""
  name='repkJByStatus'
  URL='repackjobsbystatus'
  countURL='repackjobsbystatus_count'
  title="Repack %s Jobs"
  forceRunParameter=True
  tableStepSize=100
 
  
class repkMJByStatus(repkMJ):
  """ Repack Merge Jobs by Status table: detail page"""
  name='repkMJByStatus'
  URL='repackmergejobsbystatus'
  countURL='repackmergejobsbystatus_count'
  title="Repack Merge %s Jobs"
  forceRunParameter=True
  tableStepSize=100

  
class recoJByStatus(recoJ):
  """ Reco Jobs by Status table: detail page"""
  name='recoJByStatus'
  URL='recojobsbystatus'
  countURL='recojobsbystatus_count'
  title="Reconstruction %s Jobs"
  forceRunParameter=True
  tableStepSize=100
  
class recoMJByStatus(recoMJ):
  """ Reco Merge Jobs by Status table: detail page"""
  name='recoMJByStatus'
  URL='recomergejobsbystatus'
  countURL='recomergejobsbystatus_count'
  title="Reco Merge %s Jobs"
  forceRunParameter=True
  tableStepSize=100
    
class alcaJByStatus(alcaJ):
  """ Alca Jobs by Status table: detail page"""
  name='alcaJByStatus'
  URL='alcajobsbystatus'
  countURL='alcajobsbystatus_count'
  title="Alcaskim %s Jobs"
  forceRunParameter=True
  tableStepSize=100
  
class alcaMJByStatus(alcaMJ):
  """ Alca Merge Jobs by Status table: detail page"""
  name='alcaMJByStatus'
  URL='alcamergejobsbystatus'
  countURL='alcamergejobsbystatus_count'
  title="AlcaSkim Merge %s Jobs"
  forceRunParameter=True
  tableStepSize=100
  
class DQMJByStatus(longTableContent):
  """ DQM Jobs by Status table: detail page"""
  name='DQMJByStatus'
  URL='dqmjobsbystatus'
  path='history?source='
  countURL='dqmjobsbystatus_count'
  title="DQM %s Jobs"
  colDecorators=[DQMJobId(path),RunId(),defTime(attrNames=["state_time"]),filesNum(),dataset()]
  forceRunParameter=True
  tableStepSize=100  
########################################### run page tables

###########################################################configuration
class configuration(content):
   """ superclass for configuration tables: run page"""
   URL=""
   isLongTable=False
   def updateLinks(self):
       if type(self.URL)==StringType:
         self.URL=self.URL%(str(self.run))
       elif type(self.URL)==ListType:
         newUrl=[]
         for i in range(0,len(self.URL)):
           newUrl.append(self.URL[i]%(str(self.run)))
         self.URL=newUrl
       
class ExpressConfig(configuration):
  name="ExpressConfig"
  title="Express Configuration"
  URL="express_config?run=%s&stream="
  colDecorators=[stream(),splitInProgress(),procVersion(),confUrl(),globalTag(),AMConfUrl(),expVersion(),scenario()]

class RepackConfig(configuration):
  name="RepackConfig"
  title="Repack Configuration"
  URL="repack_config?run=%s&stream="
  colDecorators=[stream(),procVersion(),repVersion()]


class GeneralConfig(configuration):
  name="GeneralConfig"
  title="Reco/Alca Configuration" 
  #URL="general_config?run=%s"
  URL=["alca_config?run=%s&","reco_config?run=%s&"]
  tableClass='ConfigMixTable'
  colDecorators=[dataset(),roReco(),roCmsswVersion(),roGlobalTag(),roConfUrl(),roProcVersion(),alAlcaskim(),alCmsswVersion(),alConfUrl(),alProcVersion(),scenario()]
  
  
#class oldGeneralConfig(configuration):
  #name="oldGeneralConfig"
  #title="Reco/Alca Configuration" 
  #URL="general_config?run=%s"
  ##URL=['alca_config?run=%s','reco_config?run=%s']
  ##tableClass='ConfigMixTable'
  #colDecorators=[dataset(),roReco(),roCmsswVersion(),roGlobalTag(),roConfUrl(),roProcVersion(),alAlcaskim(),alCmsswVersion(),alConfUrl(),alProcVersion()]

#####################################################################

class streamers(longTableContent):
  """ Streamers files table: run page"""
  name='streamers'
  #CSSclass='expressBG'
  URL='runstreamers'
  countURL='runstreamers_count'
  title="Streamers files"
  path='history?source=Streamer'
  colDecorators=[StreamerId(path),lumiId(),filesize(),events(),stream(),lfn()]
  #BUG FIX: deleted exportable() column
  
class exprTF(longTableContent):
  """ Express Temporary files table: run page"""
  name='exprTF'
  #CSSclass='expressBG'
  URL='runtempexpressfiles'
  countURL='runtempexpressfiles_count'
  title="Temporary Express files"
  path='history?source=Express'
  colDecorators=[FileId(path),filesize(),events(),lfn()]
  
class exprF(longTableContent):
  """ Express  files table: run page"""
  name='exprF'
  #CSSclass='expressBG'
  URL='runexpressfiles'
  countURL='runexpressfiles_count'
  title="Express Files"
  path='history?source=ExpressMerge'
  colDecorators=[FileId(path),filesize(),events(),lfn()]  
  isLongTable=True
    
class repkTF(longTableContent):
  """ Repack Temporary files table: run page"""
  name='repkTF'
  #CSSclass='repackBG'
  URL='runtemprepackfiles'
  countURL='runtemprepackfiles_count'
  title="Temporary Repack files"
  path='history?source=Repack'
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]
  isLongTable=True
  
class repkF(longTableContent):
  """ Repack files table: run page"""
  name='repkF'
  #CSSclass='repackBG'
  URL='runrepackfiles'
  countURL='runrepackfiles_count'
  title=" Repack files"
  path='history?source=RepackMerge'
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]
  isLongTable=True 
  
class recoTF(longTableContent):
  """ Reco Temporary files table: run page"""
  name='recoTF'
  #CSSclass='recoBG'
  URL='runtemprecofiles'
  countURL='runtemprecofiles_count'
  title="Temporary Reconstruction Files"
  path='history?source=Reco'
  colDecorators=[FileId(path),filesize(),events(),tier(),dataset(),lfn()]
  isLongTable=True
  
class recoF(longTableContent):
  """ Reco files table: run page"""
  name='recoF'
  #CSSclass='recoBG'
  URL='runrecofiles'
  countURL='runrecofiles_count'
  title="Reconstruction Files"
  path='history?source=RecoMerge'
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]
  isLongTable=True
    
class alcaTF(longTableContent):
  """ Alca Temporary files table: run page"""
  name='alcaTF'
  #CSSclass='alcaBG'
  URL='runtempalcafiles'
  countURL='runtempalcafiles_count'
  title="Temporary Alcaskim Files"
  path='history?source=Alca'
  colDecorators=[FileId(path),filesize(),events(),dataset(),skim(),lfn()]
  isLongTable=True
  
class alcaF(longTableContent):
  """ Alca files table: run page"""
  name='alcaF'
  #CSSclass='alcaBG'
  URL='runalcafiles'
  countURL='runalcafiles_count'
  title="AlcaSkim Files"
  path='history?source=AlcaMerge'
  colDecorators=[FileId(path),filesize(),events(),dataset(),skim(),lfn()]
  isLongTable=True

############################################################################ History tables
### all the table starting with H* is a history page table
class HStreamerFiles(streamers):
  title="Streamers"
#
#  every layer has 4 tables:
#  the name of every table is Express/Repack/Reco/Alca
#   jobs  ||   files   || mergeJobs  || mergeFiles
#
# the names may mislead you: because actually the files are Temporary e non Temporary ['merge' in this case is a wrong prefix]
# unfortunatly this conceptual error was made also in the query script DAS/AllHistory 
#
#############################
class HExpressJobs(exprJ):
  title=" Express Jobs"
  path='history?source=Express'
  colDecorators=[JobId(path),RunId(),defTime(),nstreamers(),status()]

class HExpressFiles(exprF):
  path='history?source=Express'
  title="Temporary Express Files"
  colDecorators=[FileId(path),filesize(),events(),lfn()]

class HExpressMergeJobs(exprMJ):
  title=" Express Merge Jobs"
  path='history?source=ExpressMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),status()]


class HExpressMergeFiles(exprTF):
  path='history?source=ExpressMerge'
  title=" Express Files"
  colDecorators=[FileId(path),filesize(),events(),lfn()]
  
#############################
class HRepackJobs(content):
  title=" Repack Jobs"
  path='history?source=Repack'
  colDecorators=[JobId(path),RunId(),defTime(),nstreamers(),stream(),status()]
  
class HRepackFiles(content):
  path='history?source=Repack'
  title="Temporary Repack Files"
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]


class HRepackMergeJobs(content):
  title=" Repack Merge Jobs"
  path='history?source=RepackMerge'
  colDecorators=[JobId(path),RunId(),filesNum(),dataset(),status()]  
  
class HRepackMergeFiles(content):
  path='history?source=RepackMerge'
  title="Repack Files"
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]
#############################
class HRecoJobs(content):
  title=" Reco Jobs"
  path='history?source=Reco'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset(),status()]
  
class HRecoFiles(content):
  path='history?source=Reco'
  title="Temporary Reco Files"
  colDecorators=[FileId(path),filesize(),events(),tier(),dataset(),lfn()]


class HRecoMergeJobs(content):
  title=" Reco Merge Jobs"
  path='history?source=RecoMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),tier(),dataset(),status()]  
  
class HRecoMergeFiles(content):
  path='history?source=RecoMerge'
  title=" Reco Files "
  colDecorators=[FileId(path),filesize(),events(),tier(),dataset(),lfn()]
#############################
class HAlcaJobs(content):
  title=" Alca Jobs"
  path='history?source=Alca'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset(),status()]
  
class HAlcaFiles(content):
  path='history?source=Alca'
  title="Temporary Alca Files"
  colDecorators=[FileId(path),filesize(),events(),skim(),dataset(),lfn()]


class HAlcaMergeJobs(content):
  title=" Alca Merge Jobs"
  path='history?source=AlcaMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),skim(),dataset(),status()]  
  
class HAlcaMergeFiles(content):
  path='history?source=AlcaMerge'
  title=" Alca Files"
  colDecorators=[FileId(path),filesize(),events(),skim(),dataset(),lfn()]


class HDQMFiles(content):
  path='history?source=DQM'
  title="DQM Files"
  colDecorators=[FileId(path),filesize(),events(),tier(),dataset(),lfn()]
class HDQMJobs(content):
  path='history?source=DQM'
  title="DQM Jobs"
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset(),status()]
#########################################################################################################################
### STATISTICS!!
class statTable(content):
  """special statistic tables: they are used in homepage,run page and detail page"""
  title="Table"
  attrStatus="status"
  attrCount="count"
  URL=""
  tableStatuses=["Acquired","Failed", "Complete", ]
  
    

class expressTable(statTable):
  title="Express stats"
  URL="statsexpress?run=%s"
  link="detail?table=exprJByStatus&run_id=%s&status=%s"
  tableStatuses=['New','Used','Abandoned','Failure','Success'] #used as display order
  
class repackTable(statTable):
  title="Repack stats"
  URL="statsrepack?run=%s"
  link="detail?table=repkJByStatus&run_id=%s&status=%s" 
  tableStatuses=['New','Used','Abandoned','Failure','Success']#used as display order
     
     
class expressMergeTable(statTable):
  title="Express Merge stats"
  #URL="statsexpressmerge?run=%s&table=%s"
  URL="statsexpressmerge?run=%s"
  link="detail?table=exprMJByStatus&run_id=%s&status=%s"
  
class repackMergeTable(statTable):
  title="Repack Merge stats"
  URL="statsrepackmerge?run=%s"
  link="detail?table=repkMJByStatus&run_id=%s&status=%s"
  
class recoTable(statTable):
  title="Reco stats"
  URL="statsreco?run=%s"
  link="detail?table=recoJByStatus&run_id=%s&status=%s" 

class recoMergeTable(statTable):
  title="Reco Merge stats"
  URL="statsrecomerge?run=%s"
  link="detail?table=recoMJByStatus&run_id=%s&status=%s"
  
class alcaTable(statTable):
  title="AlcaSkim stats"
  URL="statsalca?run=%s"
  link="detail?table=alcaJByStatus&run_id=%s&status=%s" 

class alcaMergeTable(statTable):
  title="AlcaSkim Merge stats"
  URL="statsalcamerge?run=%s"
  link="detail?table=alcaMJByStatus&run_id=%s&status=%s" 
  
class DQMTable(statTable):
  title="DQM Job stats"
  URL="statsdqm?run=%s"
  link="detail?table=DQMJByStatus&run_id=%s&status=%s"   