#!/usr/bin/env python
"""
_RuntimePostCalib_

"""

__revision__ = "$Id: RuntimePostCalib.py,v 1.6 2010/07/01 08:25:33 hartwig Exp $"
__version__ = "$Revision: 1.6 $"

import os

from ProdCommon.FwkJobRep.TaskState import TaskState
from StageOut.StageOutMgr import StageOutMgr
import StageOut.Impl


def copyFiles(target, jobSpec):
    """
    Load framework job report, check for analysis files
    and copy them to the target location.
    """
    
    files2copy = []
    
    state = TaskState(os.getcwd())
    try:
        state.loadJobReport()
    except Exception, ex:
        print "Error Reading JobReport:"
        print str(ex)
        return None
    
    report = state.getJobReport()
    if report.status != 'Success':
        print "Status of job is not 'Success'!"
        print "No stageout of files!"
        return None

    if len(report.analysisFiles) == 0:
        print "No analysis files specified in job report!"
        print "No stageout of files!"
        return None
    
    for file in report.analysisFiles:
        files2copy.append(file['FileName'])
        
    print "Files for stage out: %s" % files2copy
    
    #PromptCalib<calibTask>-Run<runNumber>-<alcaStream>-<jobID>-JobSpec.xml
    jobName = jobSpec.split('/')[-1]
    if jobName.endswith('-JobSpec.xml'):
        jobName = jobName[:-len('-JobSpec.xml')]
    if jobName.startswith('PromptCalib'):
        jobName = jobName[len('PromptCalib'):]

    # add <calibTask>_Run<runNumber> as path to target
    tmp = jobName.split('-')
    path = tmp[0] + '_' + tmp[1]
    # store file named as <alcaStream>-<jobId>
    fileName = tmp[2] + '_' + tmp[3]

    mgr = StageOutMgr()
    
    for file in files2copy:
        outfile = fileName + '.' + file.split('.')[-1]
        outpath = os.path.join(target, path, outfile)
        print '==> stage out %s to %s' % (file, outpath)
        mgr(LFN = outpath, PFN = file)

 
def main():
    """
    _main_

    Verify that the environment variable that points at the job spec
    has been set correctly and call the post processing functions.
    
    """
    print "==================PromptCalib Post Processing======================"

    # store at /store/cal/<run>/<lumisec>/<algModule>
    #          /store/cal/tmp/<node>/<algModule>
    
    target = '/store/cal/tmp/'

    jobSpec = os.environ.get("PRODAGENT_JOBSPEC", None)
    if jobSpec == None:
        msg = "Unable to find JobSpec from "
        msg += "PRODAGENT_JOBSPEC variable\n"
        msg += "Unable to proceed\n"
        raise RuntimeError, msg

    if not os.path.exists(jobSpec):
        msg = "Cannot find JobSpec file:\n %s\n" % jobSpec
        msg += "Unable to proceed\n"
        raise RuntimeError, msg

    inputReport = "./FrameworkJobReport.xml"
    if not os.path.exists(inputReport):
        msg = "ERROR: FrameworkJobReport.xml not found in directory:\n"
        msg += os.getcwd()
        raise RuntimeError, msg

    copyFiles(target, jobSpec)

    print "==================PromptCalib Post Processing Done================="

if __name__ == '__main__':
    main()
