#!/usr/bin/env python
"""
_AlgorithmInterface_

Base class for Algorithm Implementations



"""
from T0.DataStructs.StreamerJobEntity import StreamerJobEntity


class AlgorithmInterface:
    """
    _AlgorithmInterface_

    Define basic API for algorithm plugins.

    All Algorithms should inherit this interface and override __call__ to
    act as a map on a list of T0.DataStruct.TriggerSegments and produce a list
    of T0.DataStructs.StreamerJobEntity instances.
    
    Override the call method to process a list of segment
    instances. Each instance of this class will be called on the list
    of available trigger segments per cycle and needs to make a
    decision on wether to return a job definition or not.

    These instances can be used to accumulate a set of trigger
    segments during a single polling cycle. Segments can be
    accumulated by storing them into the instance and returning None.

    Call should return either None, or a list of Job defns.
    
    """
    def __init__(self):
        pass
    

    def __call__(self, *segments, **args):
        """
        _operator(*segments)_

        Analyse the list of segments, if appropriate, create and
        return a list of job definitions, otherwise return None
        Single job must be returned as a single element list.
        """
        if len(segments) == 0:
            # return empty job list
            return []
        
        raise NotImplementedError, "AlgorithmInterface.__call__ not overridden"
    

    def newJobEntity(self, *segments):
        """
        _newJobEntity_

        Factory method for new job objects, based on the content of the
        trigger segment objects provided

        No ID/Name is added, since this should be generated on insert into
        the repack_job_def table.
        
        This will be used only by scheduler. So it need to contain only
        Trigger
        
        precondition *segments shouldn't be empty
        """
        if len(segments) == 0:
            # should just through the exception?
            return None

        result = StreamerJobEntity()
        result.addTriggerSegments(segments)
        
        return result
    
