#!/usr/bin/env python
"""
_FullLumiWithEventLimit_

Algorithm that sorts the trigger segments by run, lumi and stream
and creates jobs for each combinations seperately.

Optionally one can specify a maximum number of events per output
file, a LS would then potentially split up into multiple jobs
if it has too many events for a dataset

"""

import logging
from T0.RepackerScheduler.AlgorithmInterface import AlgorithmInterface
from T0.RepackerScheduler.Registry import registerAlgorithm


class FullLumi(AlgorithmInterface):
    """
    _FullLumi_

    """
    
    def __call__(self, *segments, **args):
        """
        _operator()_

        """

        self.lumiPerRepackJob = args.get('lumiPerRepackJob', 1)

        # sort segments by run, lumi and stream
        segmentDict = {}
        for segment in segments:

            run = segment['RunID']
            stream = segment['StreamID']
            lumi = segment['LumiID']

            if segmentDict.has_key(run):
                if segmentDict[run].has_key(stream):
                    if segmentDict[run][stream].has_key(lumi):
                        segmentDict[run][stream][lumi].append(segment)
                    else:
                        segmentDict[run][stream][lumi] = [ segment ]
                else:
                    segmentDict[run][stream] = { lumi : [ segment ] }
            else:
                segmentDict[run] = { stream : { lumi : [ segment ] } }

        result = []

        # now schedule jobs
        for run in sorted(segmentDict.keys()):
            for stream in sorted(segmentDict[run].keys()):

                lumiCount = 0
                segmentList = []
                for lumi in sorted(segmentDict[run][stream].keys()):

                    lumiCount += 1
                    segmentList.extend(segmentDict[run][stream][lumi])
                    
                    if lumiCount == self.lumiPerRepackJob:
                        result.append( self.newJobEntity(*segmentList) )
                        lumiCount = 0
                        segmentList = []

        return result


registerAlgorithm(FullLumi.__name__, FullLumi)

