#!/usr/bin/env python
"""
_Registry_

Registry module for Algorithm plugins for generating repacker
job definition

"""


class Registry:
    """
    _Registry_

    Object to contain name:class references

    Algorithms: Trigger Segment selectors within a dataset
    Optimitsers: Job Combination plugins

    """
    Algorithms = {} 
    Optimisers = {}
    
    def __init__(self):
        msg = "Do not init the Registry class"
        raise RuntimeError, msg



def registerAlgorithm(algorithmName, objectRef):
    """
    _registerAlgorithm_

    Register a new Algorithm

    """
    Registry.Algorithms[algorithmName] = objectRef
    return

def registerOptimiser(optimiserName, objectRef):
    """
    _registerOptimiser_

    Register a new Optimiser

    """
    
    Registry.Optimisers[optimiserName] = objectRef
    return

def retrieveAlgorithm(algorithmName):
    """
    _retrieveAlgorithm_

    Get an instance of the selection algorithm for the name
    provided

    """
    if not Registry.Algorithms.has_key(algorithmName):
        msg = "Unknown Algorithm for RepackerScheduler: %s" % algorithmName
        raise RuntimeError, msg

    return Registry.Algorithms[algorithmName]()

def retrieveOptimiser(optimiserName):
    """
    _retrieveOptimiser_

    Get an instance of the job optimiser for the name
    provided

    """
    
    if not Registry.Optimisers.has_key(optimiserName):
        msg = "Unknown Optimiser for RepackerScheduler: %s" % optimiserName
        raise RuntimeError, msg

    return Registry.Optimisers[optimiserName]()
