#!/usr/bin/env python
"""
_ListFilesForAlcaSkim_

Oracle implementation of ListFilesForAlcaSkim.

Return a list of files that are available for processing.  This is used for
AlcaSkim in the T0.
"""

__revision__ = "$Id: ListFilesForAlcaSkim.py,v 1.5 2011/01/13 11:04:18 hufnagel Exp $"
__version__ = "$Revision: 1.5 $"

from WMCore.Database.DBFormatter import DBFormatter

from T0 import Globals

import logging

class ListFilesForAlcaSkim(DBFormatter):

    def execute(self, subscription = None, conn = None, transaction = False):

        sql = """SELECT alca_files.fileid AS file_id,
                        alca_files.parent AS file_parent,
                        alca_config.do_alca AS do_alca
                 FROM (
                   SELECT wmbs_fileset_files.fileid AS fileid,
                          MIN(wmbs_file_parent.parent) AS parent,
                          COUNT(wmbs_fileset_files.fileid),
                          COUNT(b.id)
                   FROM wmbs_fileset_files
                   LEFT OUTER JOIN wmbs_sub_files_acquired ON
                     wmbs_fileset_files.fileid = wmbs_sub_files_acquired.fileid AND
                     wmbs_sub_files_acquired.subscription = %d
                   LEFT OUTER JOIN wmbs_sub_files_complete ON
                     wmbs_fileset_files.fileid = wmbs_sub_files_complete.fileid AND
                     wmbs_sub_files_complete.subscription = %d
                   LEFT OUTER JOIN wmbs_sub_files_failed ON
                     wmbs_fileset_files.fileid = wmbs_sub_files_failed.fileid AND
                     wmbs_sub_files_failed.subscription = %d
                   INNER JOIN wmbs_file_parent ON
                     wmbs_file_parent.child = wmbs_fileset_files.fileid
                   INNER JOIN wmbs_job_assoc ON
                     wmbs_file_parent.parent = wmbs_job_assoc.fileid
                   INNER JOIN wmbs_job a ON
                     a.id = wmbs_job_assoc.job
                   LEFT OUTER JOIN wmbs_job b ON
                     b.id = a.id AND
                     b.outcome = 1
                   WHERE wmbs_fileset_files.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = %d)
                   AND wmbs_sub_files_acquired.fileid IS NULL
                   AND wmbs_sub_files_complete.fileid IS NULL
                   AND wmbs_sub_files_failed.fileid IS NULL
                   GROUP BY wmbs_fileset_files.fileid, a.jobgroup
                   HAVING COUNT(*) = COUNT(b.id)
                 ) alca_files
                 INNER JOIN wmbs_file_dataset_path_assoc ON
                   wmbs_file_dataset_path_assoc.file_id = alca_files.fileid
                 INNER JOIN dataset_path ON
                   dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = alca_files.fileid
                 INNER JOIN alca_config ON
                   alca_config.run_id = wmbs_file_runlumi_map.run AND
                   alca_config.primary_dataset_id = dataset_path.primary_dataset
                 GROUP BY alca_files.fileid, alca_files.parent, alca_config.do_alca
                 """ % (subscription,
                        subscription,
                        subscription,
                        subscription)

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)

        resultDicts = self.formatDict(results)

        fileIdList = []
        newResultDicts = []
        for result in resultDicts:
            if result['do_alca'] == 1:
                del result['do_alca']
                newResultDicts.append(result)
            else:
                fileIdList.append({ 'fileid' : result['file_id']})

        if len(fileIdList) > 0:

            sql = """INSERT INTO wmbs_sub_files_complete (subscription, fileid)
                     VALUES (%d, :fileid)""" % subscription

            self.dbi.processData(sql, fileIdList, conn = conn,
                                 transaction = transaction)

        return newResultDicts
