#!/usr/bin/env python
"""
_ListFilesForExpressMerge_

Oracle implementation of ListFilesForExpressMerge.

Return a list of files that are available for express merging.
This is used for ExpressMerge in the T0.
"""

__revision__ = "$Id: ListFilesForExpressMerge.py,v 1.4 2011/01/05 23:03:37 hufnagel Exp $"
__version__ = "$Revision: 1.4 $"

from WMCore.Database.DBFormatter import DBFormatter

class ListFilesForExpressMerge(DBFormatter):

    def execute(self, subscription = None, conn = None, transaction = False):

        mergeableFiles = []

        sql = """SELECT wmbs_fileset_files.fileid AS id,
                        dataset_run_stream_assoc.stream_id AS stream_id,
                        dataset_path.primary_dataset AS primds_id,
                        dataset_path.processed_dataset AS procds_id,
                        data_tier.name AS datatier,
                        wmbs_file_runlumi_map.run AS run,
                        wmbs_file_runlumi_map.lumi AS lumi,
                        wmbs_file_details.filesize AS filesize,
                        wmbs_fileset_files.insert_time AS lumi_done_time
                 FROM wmbs_fileset_files
                 LEFT OUTER JOIN wmbs_sub_files_acquired ON
                   wmbs_fileset_files.fileid = wmbs_sub_files_acquired.fileid AND
                   wmbs_sub_files_acquired.subscription = %d
                 LEFT OUTER JOIN wmbs_sub_files_complete ON
                   wmbs_fileset_files.fileid = wmbs_sub_files_complete.fileid AND
                   wmbs_sub_files_complete.subscription = %d
                 LEFT OUTER JOIN wmbs_sub_files_failed ON
                   wmbs_fileset_files.fileid = wmbs_sub_files_failed.fileid AND
                   wmbs_sub_files_failed.subscription = %d
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = wmbs_fileset_files.fileid
                 INNER JOIN wmbs_file_dataset_path_assoc ON
                   wmbs_file_dataset_path_assoc.file_id = wmbs_fileset_files.fileid
                 INNER JOIN dataset_path ON
                   dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
                 INNER JOIN dataset_run_stream_assoc ON
                   dataset_run_stream_assoc.primary_dataset_id = dataset_path.primary_dataset AND
                   dataset_run_stream_assoc.run_id = wmbs_file_runlumi_map.run
                 INNER JOIN data_tier ON
                   data_tier.id = dataset_path.data_tier
                 LEFT OUTER JOIN lumi_section_split_active ON
                     lumi_section_split_active.run_id = wmbs_file_runlumi_map.run AND
                     lumi_section_split_active.lumi_id = wmbs_file_runlumi_map.lumi AND
                     lumi_section_split_active.stream_id = dataset_run_stream_assoc.stream_id
                 INNER JOIN wmbs_file_details ON
                   wmbs_file_details.id = wmbs_fileset_files.fileid
                 WHERE wmbs_fileset_files.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = %d)
                 AND wmbs_sub_files_acquired.fileid IS NULL
                 AND wmbs_sub_files_complete.fileid IS NULL
                 AND wmbs_sub_files_failed.fileid IS NULL
                 AND lumi_section_split_active.run_id IS NULL
                 """ % (subscription,
                        subscription,
                        subscription,
                        subscription)

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)
        mergeableFiles.extend(self.formatDict(results))

        return mergeableFiles
