#!/usr/bin/env python
"""
_ListFilesForMerge_

Oracle implementation of ListFilesForMerge.

Return a list of files that are available for merging.
This is used for Merge in the T0.
"""

__revision__ = "$Id: ListFilesForMerge.py,v 1.10 2011/01/13 11:04:18 hufnagel Exp $"
__version__ = "$Revision: 1.10 $"

from WMCore.Database.DBFormatter import DBFormatter

import logging

class ListFilesForMerge(DBFormatter):

    def execute(self, subscription = None, conn = None, transaction = False):

        mergeableFiles = []

        # find mergeable non-RAW files (ie. with WMSBS parents)
        sql = """SELECT merge_files.fileid AS id,
                        merge_files.parent AS mergegroup,
                        wmbs_file_details.events AS events,
                        wmbs_file_details.filesize AS filesize,
                        wmbs_file_dataset_path_assoc.dataset_path_id AS datasetpath,
                        data_tier.name AS datatier,
                        wmbs_file_run_map.run AS run,
                        wmbs_file_run_map.first_lumi AS first_lumi,
                        wmbs_file_run_map.last_lumi AS last_lumi
                 FROM (
                   SELECT wmbs_fileset_files.fileid AS fileid,
                          MIN(wmbs_file_parent.parent) AS parent,
                          COUNT(wmbs_fileset_files.fileid),
                          COUNT(b.id)
                   FROM wmbs_fileset_files
                   LEFT OUTER JOIN wmbs_sub_files_acquired ON
                     wmbs_fileset_files.fileid = wmbs_sub_files_acquired.fileid AND
                     wmbs_sub_files_acquired.subscription = %d
                   LEFT OUTER JOIN wmbs_sub_files_complete ON
                     wmbs_fileset_files.fileid = wmbs_sub_files_complete.fileid AND
                     wmbs_sub_files_complete.subscription = %d
                   LEFT OUTER JOIN wmbs_sub_files_failed ON
                     wmbs_fileset_files.fileid = wmbs_sub_files_failed.fileid AND
                     wmbs_sub_files_failed.subscription = %d
                   INNER JOIN wmbs_file_parent ON
                     wmbs_file_parent.child = wmbs_fileset_files.fileid
                   INNER JOIN wmbs_job_assoc ON
                     wmbs_file_parent.parent = wmbs_job_assoc.fileid
                   INNER JOIN wmbs_job a ON
                     a.id = wmbs_job_assoc.job
                   LEFT OUTER JOIN wmbs_job b ON
                     b.id = a.id AND
                     b.outcome = 1
                   WHERE wmbs_fileset_files.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = %d)
                   AND wmbs_sub_files_acquired.fileid IS NULL
                   AND wmbs_sub_files_complete.fileid IS NULL
                   AND wmbs_sub_files_failed.fileid IS NULL
                   GROUP BY wmbs_fileset_files.fileid, a.jobgroup
                   HAVING COUNT(*) = COUNT(b.id)
                 ) merge_files
                 INNER JOIN wmbs_file_details ON
                   wmbs_file_details.id = merge_files.fileid
                 INNER JOIN (
                   SELECT wmbs_file_runlumi_map.fileid AS fileid,
                          wmbs_file_runlumi_map.run AS run,
                          MIN(wmbs_file_runlumi_map.lumi) AS first_lumi,
                          MAX(wmbs_file_runlumi_map.lumi) AS last_lumi
                   FROM wmbs_file_runlumi_map
                   GROUP BY wmbs_file_runlumi_map.fileid,
                            wmbs_file_runlumi_map.run
                 ) wmbs_file_run_map ON
                   wmbs_file_run_map.fileid = merge_files.fileid
                 INNER JOIN wmbs_file_dataset_path_assoc ON
                   wmbs_file_dataset_path_assoc.file_id = merge_files.fileid
                 INNER JOIN dataset_path ON
                   dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
                 INNER JOIN data_tier ON
                   data_tier.id = dataset_path.data_tier AND
                   data_tier.name != 'RAW'
                 """ % (subscription,
                        subscription,
                        subscription,
                        subscription)

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)
        mergeableFiles.extend(self.formatDict(results))

        # find mergeable RAW files
        sql = """SELECT wmbs_fileset_files.fileid AS id,
                        streamer_file_run_map.first_lumi AS mergegroup,
                        wmbs_file_details.events AS events,
                        wmbs_file_details.filesize AS filesize,
                        wmbs_file_dataset_path_assoc.dataset_path_id AS datasetpath,
                        data_tier.name AS datatier,
                        streamer_file_run_map.run AS run,
                        streamer_file_run_map.first_lumi AS first_lumi,
                        streamer_file_run_map.last_lumi AS last_lumi
                 FROM wmbs_fileset_files
                 LEFT OUTER JOIN wmbs_sub_files_acquired ON
                   wmbs_fileset_files.fileid = wmbs_sub_files_acquired.fileid AND
                   wmbs_sub_files_acquired.subscription = %d
                 LEFT OUTER JOIN wmbs_sub_files_complete ON
                   wmbs_fileset_files.fileid = wmbs_sub_files_complete.fileid AND
                   wmbs_sub_files_complete.subscription = %d
                 LEFT OUTER JOIN wmbs_sub_files_failed ON
                   wmbs_fileset_files.fileid = wmbs_sub_files_failed.fileid AND
                   wmbs_sub_files_failed.subscription = %d
                 INNER JOIN (
                   SELECT wmbs_file_parent.child AS fileid,
                          streamer.run_id AS run,
                          MIN(streamer.lumi_id) AS first_lumi,
                          MAX(streamer.lumi_id) AS last_lumi
                   FROM streamer
                   LEFT OUTER JOIN lumi_section_split_active ON
                     lumi_section_split_active.run_id = streamer.run_id AND
                     lumi_section_split_active.lumi_id = streamer.lumi_id AND
                     lumi_section_split_active.stream_id = streamer.stream_id
                   INNER JOIN wmbs_file_parent ON
                     wmbs_file_parent.parent = streamer.streamer_id
                   WHERE lumi_section_split_active.run_id IS NULL
                   GROUP BY wmbs_file_parent.child, streamer.run_id
                 ) streamer_file_run_map ON
                   streamer_file_run_map.fileid = wmbs_fileset_files.fileid
                 INNER JOIN wmbs_file_details ON
                   wmbs_file_details.id = wmbs_fileset_files.fileid
                 INNER JOIN wmbs_file_dataset_path_assoc ON
                   wmbs_file_dataset_path_assoc.file_id = wmbs_fileset_files.fileid
                 INNER JOIN dataset_path ON
                   dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
                 INNER JOIN data_tier ON
                   data_tier.id = dataset_path.data_tier
                 WHERE wmbs_fileset_files.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = %d)
                 AND wmbs_sub_files_acquired.fileid IS NULL
                 AND wmbs_sub_files_complete.fileid IS NULL
                 AND wmbs_sub_files_failed.fileid IS NULL
                 """ % (subscription,
                        subscription,
                        subscription,
                        subscription)

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)
        mergeableFiles.extend(self.formatDict(results))

        #
        # sort by run, datatier and datasetpath
        #
        mergeableFilesDict = {}
        for f in mergeableFiles:

            run = f['run']
            datatier = f['datatier']
            datasetpath = f['datasetpath']

            if not mergeableFilesDict.has_key(run):
                mergeableFilesDict[run] = {}

            if not mergeableFilesDict[run].has_key(datatier):
                mergeableFilesDict[run][datatier] = { 'closeout' : False, 'datasetpaths' : {} }

            if not mergeableFilesDict[run][datatier]['datasetpaths'].has_key(datasetpath):
                mergeableFilesDict[run][datatier]['datasetpaths'][datasetpath] = {}

            # convenience reference
            mergeUnits = mergeableFilesDict[run][datatier]['datasetpaths'][datasetpath]

            # add files to matching merge unit or create new one
            if mergeUnits.has_key(f['mergegroup']):
                mergeUnit = mergeUnits[f['mergegroup']]
                mergeUnit['FILES'].append(f),
                mergeUnit['SIZE'] += f['filesize']
                mergeUnit['EVENTS'] += f['events']
                if f['first_lumi'] < mergeUnit['FIRSTLUMI']:
                    mergeUnit['FIRSTLUMI'] = f['first_lumi']
                if f['last_lumi'] > mergeUnit['LASTLUMI']:
                    mergeUnit['LASTLUMI'] = f['last_lumi']
            else:
                mergeUnit = {}
                mergeUnit['RUN'] = run
                mergeUnit['EVENTS'] = f['events']
                mergeUnit['SIZE'] = f['filesize']
                mergeUnit['FIRSTLUMI'] = f['first_lumi']
                mergeUnit['LASTLUMI'] = f['last_lumi']
                mergeUnit['FILES'] = [f]
                mergeUnits[f['mergegroup']] = mergeUnit

        #
        # Now figure out closeout condition based on run status and datatier
        #
        if len(mergeableFilesDict) > 0:

            sql = """SELECT run.run_id AS run,
                            run_status.status AS status
                     FROM run
                     INNER JOIN run_status ON
                       run_status.id = run.run_status
                     WHERE run.run_id = :RUN_ID
                     """

            bindVarList = []
            for run in mergeableFilesDict.keys():
                bindVarList.append( { 'RUN_ID' : run } )

            results = self.dbi.processData(sql, bindVarList, conn = conn,
                                       transaction = transaction)
            for result in self.formatDict(results):

                run = result['run']
                status = result['status']

                for datatier in mergeableFilesDict[run].keys():

                    if datatier == "RAW":
                        if status in ["CloseOutRepackMerge", "CloseOutPromptReco",
                                      "CloseOutRecoMerge", "CloseOutAlcaSkim",
                                      "CloseOutAlcaSkimMerge", "CloseOutExport",
                                      "CloseOutT1Skimming", "Complete"]:
                            mergeableFilesDict[run][datatier]['closeout'] = True

                    elif datatier == "RECO" or datatier == "AOD" or datatier == "DQM":
                        if status in ["CloseOutRecoMerge", "CloseOutAlcaSkim",
                                      "CloseOutAlcaSkimMerge", "CloseOutExport",
                                      "CloseOutT1Skimming", "Complete"]:
                            mergeableFilesDict[run][datatier]['closeout'] = True

                    elif datatier == "ALCARECO" or datatier == "ALCAPROMPT":
                        if status in ["CloseOutAlcaSkimMerge", "CloseOutExport",
                                      "CloseOutT1Skimming", "Complete"]:
                            mergeableFilesDict[run][datatier]['closeout'] = True

        return mergeableFilesDict
