#!/usr/bin/env python
"""
_ListFilesForPromptCalib_

Oracle implementation of ListFilesForPromptCalib.

Return a list of files that are available for processing.  This is used for prompt
calibration in the T0.
"""

__revision__ = "$Id: ListFilesForPromptCalib.py,v 1.3 2010/04/29 07:12:07 hufnagel Exp $"
__version__ = "$Revision: 1.3 $"

from WMCore.Database.DBFormatter import DBFormatter

class ListFilesForPromptCalib(DBFormatter):

    sql = """SELECT a.fileid as file_id, d.primary_dataset as primds_id,
                    e.id as procds_id, Min(f.lumi) as lumi_min,
                    Max(f.lumi) as lumi_max, g.split_algo as split_algo, 
                    g.split_param as split_param, g.config_url
             FROM wmbs_fileset_files a
             INNER JOIN wmbs_file_dataset_path_assoc c
             ON c.file_id = a.fileid
             INNER JOIN dataset_path d
             ON d.id = c.dataset_path_id
             INNER JOIN processed_dataset e
             ON e.id = d.processed_dataset
             INNER JOIN wmbs_file_runlumi_map f
             ON f.fileid = a.fileid
             INNER JOIN calib_config g
             ON ( g.run_id = f.run AND
                  g.primary_dataset_id = d.primary_dataset AND
                  g.processed_dataset = e.name)
             WHERE a.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = :subscription)
             AND NOT EXISTS
                ( SELECT 1 FROM wmbs_job w
                  INNER JOIN wmbs_jobgroup x
                  ON x.id = w.jobgroup
                  INNER JOIN wmbs_job_assoc y
                  ON y.job = w.id
                  WHERE x.subscription = :subscription
                  AND y.fileid = a.fileid )
             AND NOT EXISTS
                ( SELECT 1 FROM wmbs_sub_files_complete compl
                  WHERE compl.subscription = :subscription
                  AND compl.fileid = a.fileid )
             AND NOT EXISTS
                ( SELECT 1 FROM wmbs_sub_files_failed fail
                  WHERE fail.subscription = :subscription
                  AND fail.fileid = a.fileid )
             AND NOT EXISTS
                ( SELECT 1 FROM wmbs_sub_files_acquired acqu
                  WHERE acqu.subscription = :subscription
                  AND acqu.fileid = a.fileid )   
             GROUP BY a.fileid, d.primary_dataset, e.id,
                  g.split_algo, g.split_param, g.config_url
             ORDER BY d.primary_dataset, e.id, g.config_url, min(f.lumi)
    """
    
    def execute(self, subscription = None, conn = None, transaction = False):
        results = self.dbi.processData(self.sql, {"subscription": subscription},
                                       conn = conn, transaction = transaction)
        return self.formatDict(results)
