#!/usr/bin/env python
"""
_ListFilesForReco_

Oracle implementation of ListFilesForReco.

Return a list of files that are available for processing.  This is used for
PromptReco in the T0 and will query T0 specific tables to determine if a run
has been released for PromptReco.
"""

__revision__ = "$Id: ListFilesForReco.py,v 1.9 2010/08/11 13:25:30 hufnagel Exp $"
__version__ = "$Revision: 1.9 $"

from WMCore.Database.DBFormatter import DBFormatter

import logging

class ListFilesForReco(DBFormatter):

    def execute(self, subscription = None, conn = None, transaction = False):

        sql = """SELECT wmbs_fileset_files.fileid AS file_id,
                        reco_config.reco_split AS reco_split,
                        wmbs_file_details.events AS events,
                        CASE
                          WHEN reco_config.run_id IS NULL THEN 0
                          ELSE reco_config.do_reco
                        END AS do_reco
                 FROM wmbs_fileset_files
                 INNER JOIN wmbs_file_dataset_path_assoc ON
                   wmbs_file_dataset_path_assoc.file_id = wmbs_fileset_files.fileid
                 INNER JOIN dataset_path ON
                   dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
                 INNER JOIN primary_dataset ON
                   primary_dataset.id = dataset_path.primary_dataset
                 INNER JOIN (
                   SELECT wmbs_file_runlumi_map.fileid AS fileid,
                          wmbs_file_runlumi_map.run AS run
                   FROM wmbs_file_runlumi_map
                   GROUP BY wmbs_file_runlumi_map.fileid,
                            wmbs_file_runlumi_map.run
                 ) wmbs_file_run_map ON
                   wmbs_file_run_map.fileid = wmbs_fileset_files.fileid
                 LEFT OUTER JOIN reco_config ON
                   reco_config.run_id = wmbs_file_run_map.run AND
                   reco_config.primary_dataset_id = dataset_path.primary_dataset
                 INNER JOIN run ON
                   run.run_id = wmbs_file_run_map.run AND
                   run.reco_started = 1
                 INNER JOIN wmbs_file_details ON
                   wmbs_file_details.id = wmbs_fileset_files.fileid
                 LEFT OUTER JOIN wmbs_sub_files_acquired ON
                   wmbs_sub_files_acquired.fileid = wmbs_fileset_files.fileid AND
                   wmbs_sub_files_acquired.subscription = %d
                 LEFT OUTER JOIN wmbs_sub_files_complete ON
                   wmbs_sub_files_complete.fileid = wmbs_fileset_files.fileid AND
                   wmbs_sub_files_complete.subscription = %d
                 LEFT OUTER JOIN wmbs_sub_files_failed ON
                   wmbs_sub_files_failed.fileid = wmbs_fileset_files.fileid AND
                   wmbs_sub_files_failed.subscription = %d
                 WHERE wmbs_fileset_files.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = %d)
                 AND wmbs_sub_files_acquired.fileid IS NULL
                 AND wmbs_sub_files_complete.fileid IS NULL
                 AND wmbs_sub_files_failed.fileid IS NULL
                 """ % (subscription,
                        subscription,
                        subscription,
                        subscription)

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)

        resultDicts = self.formatDict(results)

        fileIdList = []
        newResultDicts = []
        for result in resultDicts:
            if result['do_reco'] == 1:
                del result['do_reco']
                newResultDicts.append(result)
            else:
                fileIdList.append({ 'fileid' : result['file_id']})

        if len(fileIdList) > 0:

            sql = """INSERT INTO wmbs_sub_files_complete (subscription, fileid)
                     VALUES (%d, :fileid)""" % subscription

            self.dbi.processData(sql, fileIdList, conn = conn,
                                 transaction = transaction)

        return newResultDicts
