#!/usr/bin/env python
"""
_ListFilesForRepack_

Oracle implementation of ListFilesForRepack.

Return a list of files that are available for repacking.
"""

__revision__ = "$Id: ListFilesForRepack.py,v 1.3 2011/01/20 14:57:51 hufnagel Exp $"
__version__ = "$Revision: 1.3 $"

from WMCore.Database.DBFormatter import DBFormatter

import logging

class ListFilesForRepack(DBFormatter):

    def execute(self, conn = None, transaction = False):

        sql = """SELECT run_id FROM run
                 WHERE run_status = (SELECT id FROM run_status WHERE status = 'Active')"""

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)

        activeRuns = []
        for result in self.formatDict(results):
            activeRuns.append(result['run_id'])

        sql = """SELECT streamer.streamer_id AS id,
                        wmbs_file_runlumi_map.run AS run,
                        wmbs_file_runlumi_map.lumi AS lumi,
                        wmbs_file_details.events AS events,
                        wmbs_file_details.filesize AS filesize,
                        streamer.stream_id AS stream_id
                  FROM streamer
                  INNER JOIN run_stream_style_assoc ON
                    run_stream_style_assoc.run_id = streamer.run_id AND
                    run_stream_style_assoc.stream_id = streamer.stream_id AND
                    run_stream_style_assoc.style_id = (SELECT id FROM processing_style WHERE name = 'Bulk')
                  INNER JOIN lumi_section_closed ON
                    lumi_section_closed.run_id = streamer.run_id AND
                    lumi_section_closed.lumi_id = streamer.lumi_id AND
                    lumi_section_closed.stream_id = streamer.stream_id AND
                    lumi_section_closed.close_time > 0
                  INNER JOIN wmbs_file_runlumi_map ON
                    wmbs_file_runlumi_map.fileid = streamer.streamer_id
                  INNER JOIN wmbs_file_details ON
                    wmbs_file_details.id = streamer.streamer_id
                  WHERE checkForZeroState(streamer.used) = 0
                  """

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)

        #
        # sort by run and stream_id, also add closeout state
        #
        streamerDict = {}
        for result in self.formatDict(results):

            run = result['run']
            stream_id = result['stream_id']
            lumi = result['lumi']

            if not streamerDict.has_key(run):
                streamerDict[run] = { 'closeout' : False, 'streams' : {} }

            if not streamerDict[run]['streams'].has_key(stream_id):
                streamerDict[run]['streams'][stream_id] = {}

            if not streamerDict[run]['streams'][stream_id].has_key(lumi):
                streamerDict[run]['streams'][stream_id][lumi] = []

            streamerDict[run]['streams'][stream_id][lumi].append(result)

        # now checks for run in closeout
        for run in streamerDict.keys():
            if run not in activeRuns:
                streamerDict[run]['closeout'] = True

        return streamerDict
