#!/usr/bin/env python
"""
_ListCleanup_

Queries used for cleanup
"""

__version__ = "$Revision: 1.9 $"
__revision__ = "$Id: ListCleanup.py,v 1.9 2011/01/20 14:57:51 hufnagel Exp $"

import logging

from T0 import Globals

def getDeletableStreamers(dbConn):
    """
    _getDeletableStreamers_

    returns dictionary with id as key and lfn as value

    """
    sqlQuery = """SELECT streamer.streamer_id,
                         wmbs_file_details.lfn
                  FROM streamer
                  INNER JOIN wmbs_file_details ON
                    wmbs_file_details.id = streamer.streamer_id
                  WHERE checkForZeroOneState(streamer.deleted) = 1
                  AND ROWNUM <= 100000
                  """

    dbConn.execute(sqlQuery, useTransaction = False, returnCursor = True)
    cursors = dbConn.queryResult

    streamers = {}
    for cursor in cursors:
        while True:
            results = cursor.fetchmany(size = 1000)
            if not results:
                break
            for result in results:
                streamers[result[1]] = result[0]

    return streamers

def getDeletableFiles(dbConn):
    """
    _getDeletableFiles_

    returns dictionary with id as key and lfn as value
    separated by temp and merged files

    """
    deleteSubscription = Globals.retrieveDeleteSubscription()

    sqlQuery = """SELECT wmbs_fileset_files.fileid, wmbs_file_details.lfn
                  FROM wmbs_fileset_files
                  LEFT OUTER JOIN wmbs_sub_files_complete ON
                    wmbs_sub_files_complete.fileid = wmbs_fileset_files.fileid AND
                    wmbs_sub_files_complete.subscription = %d
                  INNER JOIN wmbs_file_details ON
                    wmbs_file_details.id = wmbs_fileset_files.fileid
                  WHERE wmbs_fileset_files.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = %d)
                  AND wmbs_sub_files_complete.fileid IS NULL
                  AND ROWNUM <= 100000
                  """ % (deleteSubscription['id'],
                         deleteSubscription['id'])

    dbConn.execute(sqlQuery, useTransaction = False, returnCursor = True)
    cursors = dbConn.queryResult

    wmbsFiles = { 'TEMP' : {}, 'MERGED' : {} }
    for cursor in cursors:
        while True:
            results = cursor.fetchmany(size = 1000)
            if not results:
                break
            for result in results:
                if result[1].startswith("/store/t0temp/"):
                    wmbsFiles['TEMP'][result[1]] = result[0]
                else:
                    wmbsFiles['MERGED'][result[1]] = result[0]

    return wmbsFiles
