#!/usr/bin/env python
"""
_ListTriggerSegments_

Functions to query the trigger_segment table in T0AST.
"""

__revision__ = "$Id: ListTriggerSegments.py,v 1.23 2009/09/03 07:27:56 hufnagel Exp $"
__version__ = "$Revision: 1.23 $"

def getNewTriggerSegments(dbConn):
    """
    _getNewTriggerSegments_

    Retrieve trigger_segments from T0AST that have status 'New'
    """
    sqlQuery = """SELECT a.primary_dataset_id, b.stream_id, a.streamer_id, b.lumi_id, b.run_id, a.segment_size
                  FROM trigger_segment a
                  INNER JOIN streamer b
                  ON b.streamer_id = a.streamer_id
                  WHERE status = (SELECT id FROM trigger_segment_status WHERE status = 'New')"""

    dbConn.execute(sqlQuery)
    results = dbConn.fetchall()

    segmentList = []
    for row in results:
        segment = {}
        segment["DatasetID"] = row[0]
        segment["StreamID"] = row[1]
        segment["StreamerID"] = row[2]
        segment["LumiID"] = row[3]
        segment["RunID"] = row[4]
        segment["SegmentSize"] = row[5]
        segmentList.append(segment)

    return segmentList

def countTriggerSegmentsByStatus(dbConn, runNumber, statusList):
    """
    _countTriggerSegmentsByStatus_

    Count the number of trigger_segments from a particular run with a particular
    status.  Note that each entry in the status list must correspond with a row
    in the trigger_segment_status table.
    """
    if type(statusList) != list:
        statusList = [statusList]
    
    sqlQuery = """SELECT COUNT(*) FROM trigger_segment a
                  INNER JOIN streamer b
                  ON b.streamer_id = a.streamer_id
                  WHERE b.run_id = :p_1
                  AND status = (SELECT id FROM trigger_segment_status WHERE status = :p_2)"""

    bindVars = []
    for status in statusList:
        bindVars.append({"p_1": runNumber, "p_2": status})

    dbConn.execute(sqlQuery, bindVars)
    results = dbConn.fetchall()

    total = 0
    for result in results:
        total += int(result[0])

    return total

def countTriggerSegmentsByRun(dbConn, runNumber):
    """
    _countTriggerSegmentsByRun_

    Count the number of trigger segments that exist in T0AST for a
    particular run.
    """
    sqlQuery = """SELECT COUNT(*) FROM trigger_segment a
                  INNER JOIN streamer b
                  ON b.streamer_id = a.streamer_id
                  WHERE b.run_id = :p_1"""

    bindVars = {"p_1": runNumber}
    dbConn.execute(sqlQuery, bindVars)
    result = dbConn.fetchall()

    while type(result) != int:
        result = result[0]

    return result

