#!/usr/bin/env python
"""
_InsertStreamer_

Functions for updating information in the streamer table.
"""

__revision__ = "$Id: InsertStreamer.py,v 1.20 2010/06/18 10:25:32 hufnagel Exp $"
__version__ = "$Revision: 1.20 $"

import time
import logging

def updateSplitStreamers(dbConn, streamerIDs):
    """
    _updateSplitStreamers_

    Update the splitable column in the streamer table to be 0 for streamer
    that have been split into trigger segments.  The streamerIDs parameter
    must be a list of streamer ids
    """
    if len(streamerIDs) == 0:
        return
    
    sqlQuery = """UPDATE streamer
                  SET used = 1
                  WHERE streamer_id = :p_1"""

    bindVarList = []
    for streamerID in streamerIDs:
        bindVarList.append( {'p_1': streamerID} )
    dbConn.execute(sqlQuery, bindVarList)

    return

def insertStreamers(dbConn, streamerList):
    """
    _insertStreamers_
    
    This function is used by Tier0Accountant to insert data in streamer table
    when MC Tier0 Feeder is enabled. In normal operation the data comes from 
    storage manager
    """
    if type(streamerList) != list:
        streamerList = [streamerList]
    
    sqlQuery = """INSERT INTO streamer (streamer_id, run_id, lumi_id, 
                        insert_time, filesize, events, lfn, stream_id)
                  VALUES (streamer_SEQ.nextval, :run_id, :lumi_id,
                          :insert_time, :filesize, :events, :lfn,
                          (SELECT id FROM  stream WHERE name = :stream))"""

    bindVars = []
    currentTime = int(time.time())
    for streamer in streamerList:
        bindVar = {}
        bindVar['run_id'] = streamer['RunNumber']
        bindVar['lumi_id'] = streamer['LumiID']
        bindVar['filesize'] = streamer['FileSize']
        bindVar['events'] = streamer['Events']
        bindVar['lfn'] = streamer['LFN']
        bindVar['stream'] = streamer['Stream']
        bindVar['insert_time'] = int(time.time())
        bindVars.append(bindVar)

    dbConn.execute(sqlQuery, bindVars)
    
    return

