#!/usr/bin/env python
"""
_InsertTriggerSegment_

Functions for manipulating data in the trigger_segment table.
"""

__revision__ = "$Id: InsertTriggerSegment.py,v 1.5 2009/09/03 07:27:55 hufnagel Exp $"
__version__ = "$Revision: 1.5 $"

def insertTriggerSegments(dbConn, triggerSegments):
    """
    _insertTriggerSegments_

    Insert trigger segments into the trigger_segment table.  Note that the
    triggerSegments parameter must be a list of dictionaries.  Each dictionary
    must have the following keys:
      dataset_id
      streamer_id
      lumi_id
      run_id
      segment_size
      status
    """
    if type(triggerSegments) != list:
        triggerSegments = [triggerSegments]
    
    sqlQuery = """INSERT INTO trigger_segment
                  (PRIMARY_DATASET_ID, STREAMER_ID, SEGMENT_SIZE, STATUS)
                  VALUES (:p_1, :p_2, :p_3, :p_4)"""

    bindVars = []
    for segment in triggerSegments:
        bindVars.append( { "p_1" : segment['dataset_id'],
                           "p_2" : segment['streamer_id'],
                           "p_3" : segment['segment_size'],
                           "p_4" : segment['status'] } )
    dbConn.execute(sqlQuery, bindVars)
    return

def updateTriggerSegmentStatus(dbConn, streamerIDs, datasetIDs, status):
    """
    _updateTriggerSegmentStatus_

    Update the status of a trigger segment.  The status parameter must be a
    string that corresponds to a row in the trigger_segment_status table.

    """
    sqlQuery = """UPDATE trigger_segment
                  SET status = (SELECT id FROM trigger_segment_status WHERE status = :p_1)
                  WHERE streamer_id = :p_2
                  AND primary_dataset_id = :p_3"""

    bindVars = []
    for streamerID in streamerIDs:
        for datasetID in datasetIDs:
            bindVars.append( { "p_1": status,
                               "p_2": streamerID,
                               "p_3": datasetID } )
    dbConn.execute(sqlQuery, bindVars)
    return

def deleteTriggerSegments(dbConn, streamerIDs, datasetIDs):
    """
    _deleteTriggerSegments_

    Delete all trigger segments for the provided
    dataset and streamers ids

    """
    sqlQuery = """DELETE from trigger_segment
                  WHERE streamer_id = :p_1
                  AND primary_dataset_id = :p_2"""

    bindVars = []
    for streamerID in streamerIDs:
        for datasetID in datasetIDs:
            bindVars.append( { "p_1": streamerID,
                               "p_2": datasetID } )
    dbConn.execute(sqlQuery, bindVars)
    return
