from types import *
import json as json
import urllib2,threading
from time import sleep

class DASManager():
  """
   This class manage all the parallel connections to the DAS Server.  
  """
  reconnectionsDefaultNumber=3
  
  def __init__(self,config):
    self.config=config
    self.resDict={}
    self.errors=[]
    
  def calls(self,urls):
    """
    This method calls a list of urls and return a dict with urls as keys 
    Pattern: Observer class on DASClass
    """
    count=self.reconnectionsDefaultNumber
    while len(self.resDict)==0 or len(self.errors)>0:
      #try to connect if the result is void OR there are errors
      if len(self.errors)>0: urls=self.errors
      self.errors=[]
      count-=1
      threads=[]
      for url in urls:
        self.config.threadSemaphore.acquire()
        call = DASCall(self.config,url)
        call.addObserver(self)
        threads.append(call)
        call.start()
      for c in threads:
        c.join()
        
      if len(self.errors)>0:
        if count>0:
          print "trying new connection..."
          sleep(5)
        else:
          #if too many errors has happened raise an exception!
          print "URL errors="+str(self.errors)
          raise DASException('Impossibile to connect')
      
    return self.resDict
    
  def update(self,call):
    self.config.threadSemaphore.release()
    if call.status==0:
     self.resDict.update({call.url:call.res})
    else: 
      self.errors.append(call.url)
     
    
    


class DASCall(threading.Thread):
  """
  This class represent a single DAS url connection, it reads the Json object and return a python list
  Pattern: Observable class, Runnable object
  """
  def __init__(self,config,url):
    self.url=url
    self._observers=[]
    self.config=config
    #print "Calling: "+str(url)
    self.res=[]
    self.status=0
    threading.Thread.__init__(self)
  
  def addObserver(self, observer):
   if not observer in self._observers:
      self._observers.append(observer)
      
  def run(self):
    self.req = urllib2.Request(self.config.DASUrl+self.url) 
    ## set the header to accept json data
    self.req.add_header("Content-type","application/x-www-form-urlencoded")
    self.req.add_header("Accept","application/json")
    try: 
      self.response = urllib2.urlopen(self.req)
      self.jsonRaw = self.response.read()
      jsonObj = json.loads(self.jsonRaw)
      #print "data=="+str(jsonObj)
     #data MUST be a list of dictionaries
      if type(jsonObj) is ListType:
        self.res=jsonObj
        self.sInfo={}
      else: 
        self.res=[jsonObj,]
    except Exception, e:
      self.status=1
      self.res=[str(e)]
    for observer in self._observers:
      observer.update(self)

class DASException(Exception):
  pass