#!/usr/bin/env python
"""
_Merge_

Job handler for Merge jobs.
"""

__revision__ = "$Id: Merge.py,v 1.33 2011/01/19 21:23:47 hufnagel Exp $"
__version__ = "$Revision: 1.33 $"

import logging

from T0 import Globals

from T0.Tier0Accountant.WMBSBaseHandler import WMBSBaseHandler

class Merge(WMBSBaseHandler):
    """
    _RepackMerge_

    Job handler for Merge jobs.

    """
    def handleSuccessfulJob(self, jobInfo, jobFiles):
        """
        _handleSuccessfulJob_

        sanity check on having a single output file
        call base handler

        """
        logging.debug("MergeHandler handles successful job id:%s"
                      % jobInfo["JOB_IDENTIFIER"])
        
        if len(jobFiles) > 1:
            logging.error("WARNING: Merge job with more than one output file: %s" % jobFiles)

        # update merge flag
        for jobFile in jobFiles:
            jobFile["MERGED"] = True
            
        return WMBSBaseHandler.handleSuccessfulJob(self, jobInfo, jobFiles)
        
    
    def getFilesetListByFile(self, t0astFile):
        """
        _getFilesetListByFile_ 

        retrieve the fileset list for next processing steps

        all files are uploaded to DBS

        RAW also is reconstructed
        DQM also is DQM harvested
        ALCAPROMPT also is Alca harvested

        """
        filesetList = []

        filesetList.append( Globals.retrieveDBSUploadSubscription()['fileset'].id )

        if t0astFile["DATA_TIER"] == "RAW":
            filesetList.append( Globals.retrieveRecoSubscription()['fileset'].id )
        elif t0astFile["DATA_TIER"] == "DQM":
            filesetList.append( Globals.retrieveDQMSubscription()['fileset'].id )
        elif t0astFile["DATA_TIER"] == "ALCAPROMPT":
            filesetList.append( Globals.retrieveAlcaHarvestSubscription()['fileset'].id )

        return filesetList
