#!/usr/bin/env python
"""
_PromptReco_

Job handler for PromptReco jobs.
"""

__revision__ = "$Id: PromptReco.py,v 1.32 2011/01/19 21:23:47 hufnagel Exp $"
__version__ = "$Revision: 1.32 $"

import logging

from T0 import Globals

from T0.Tier0Accountant.WMBSBaseHandler import WMBSBaseHandler
from T0.State.Database.Writer import InsertRunConfig

class PromptReco(WMBSBaseHandler):
    """
    _PromptReco_

    Job handler for PromptReco jobs.

    """
    def handleSuccessfulJob(self, jobInfo, jobFiles):
        """
        _handleSuccessfulJob_

        call base handler
        update pset hash if that suceeds

        """
        logging.debug("PromptRecoHandler handles successful job id:%s"  
                      % jobInfo["JOB_IDENTIFIER"])
        
        wmbsJobInfo = WMBSBaseHandler.handleSuccessfulJob(self, jobInfo, jobFiles)

        if wmbsJobInfo != None:

            # update the PSetHash, only do this once
            if len(jobFiles) > 0:

                t0astFile = jobFiles[0]
                InsertRunConfig.updatePSetHash(self.t0astDBConn,
                                               "reco_config",
                                               t0astFile["RUN_ID"],
                                               t0astFile["DATASET_ID"],
                                               t0astFile["PSET_HASH"])

        return wmbsJobInfo


    def getFilesetListByFile(self, t0astFile):
        """
        _getFilesetListByFile_ 
        
        retrieve the fileset list for next processing steps

        """
        filesetList = []

        if t0astFile["DATA_TIER"] == "ALCARECO":
            filesetList.append( Globals.retrieveAlcaSubscription()['fileset'].id )
        else:
            filesetList.append( Globals.retrieveMergeSubscription()['fileset'].id )

        return filesetList
