import logging
from T0.State.Database.Writer import InsertPerformanceData

def storePerformanceReport(dbConn, jobReport, jobInfo):
    """
    _storePerformanceReport_
    
    ## Metrics with this structure (summaries) are in jobReport.performance.summaries
    ## <PerformanceReport>
    ##  <PerformanceSummary Metric="...">
    ##
    ## Metrics with this structure (Modules) are in jobReport.performance.modules
        ## <PerformanceReport>
        ##  <PerformanceSummary Metric="..."  Module="..." >
    """    
    # get performance summaries from jobReport
    if jobInfo["JOB_TYPE"] == "DQMHarvest":
        logging.info("Tier0 doesn't do a bookeeping for DQMHarvest jobs")
        return
        
    try:
        performanceSummaries = jobReport.performance.summaries
        #TODO: need module as well?
        #performanceModules = jobReport.performance.modules
    except AttributeError, ex:
        logging.error("No performance data is reported in FWJR.")
        return
    
    # make one transaction 
    dbConn.beginTransaction()
    cpuID = None
    if  performanceSummaries.has_key("SystemCPU"):
        # update System CPU data
        cpuID =  _updateSystemCPUData(performanceSummaries["SystemCPU"])
    
    if performanceSummaries.has_key("Timing"):
        # update Timing related information
        _updateJobPerfData(performanceSummaries["Timing"], jobReport, 
                          jobInfo["JOB_IDENTIFIER"], cpuID)

    dbConn.commit()
    return

def _updateSystemCPUData(dbConn, perfSummary, jobID):
    """
    _updateSystemCPUData_
    
    insert system CPU info to T0AST
    TODO: mapping between bind variable and key value occurs here,
          Maybe it is better in Database module (i.e insert...)
    """
    cpuInfo = {} 
    cpuInfo['cpu_model'] = getValue(perfSummary, "CPUModels")
    cpuInfo['avg_core_speed'] = getValue(perfSummary, "averageCoreSpeed")
    cpuInfo['total_cores'] = getValue(perfSummary, "totalCPUs")
    cpuID = InsertPerformanceData.insertSystemCPUInfo(dbConn, cpuInfo)
    return cpuID

def _updateJobPerfData(dbConn, perfSummary, jobReport, jobID, jobType, cpuID):
    """
    _uupdateJobPerfData_
    
    insert job timing info to T0AST
    TODO: mapping between bind variable and key value occurs here,
          Maybe it is better in Database module (i.e insert...)
    """
    perfData = {} 
    perfData['cpu_id'] = cpuID
    perfData['avg_event_cpu'] = getValue(perfSummary, "AvgEventCPU")
    perfData['max_event_cpu'] = getValue(perfSummary, "MaxEventCPU")
    perfData['min_event_cpu'] = getValue(perfSummary, "MinEventCPU")
    perfData['avg_event_time'] = getValue(perfSummary, "AvgEventTime")
    perfData['max_event_time'] = getValue(perfSummary, "MaxEventTime")
    perfData['min_event_time'] = getValue(perfSummary, "MinEventTime")
    perfData['total_event_cpu'] = getValue(perfSummary, "TotalEventCPU")
    perfData['total_job_cpu'] = getValue(perfSummary, "TotalJobCPU")
    perfData['total_job_time'] = getValue(perfSummary, "TotalJobTime")
    
    numEvents = 0
    if hasattr(jobReport, "inputFiles") and len(jobReport.inputFiles) > 0:
        for file in jobReport.inputFiles:
            if file.has_key("EventsRead") and file["EventsRead"] != None:
                numEvents += int(file["EventsRead"])
    else:
        #TODO: not sure it will be translated to Null value in db
        numEvents = None
    
    perfData['num_events'] = numEvents
        
    InsertPerformanceData.insertPerfData(dbConn, jobID, jobType, perfData)
    
    return

def getValue(dictModule, key, valueList=['nan', 'inf'], defaultValue=None):
    """
    _getValue_
    
    convert to a default value for the key if the value is in given value list
    """
    value = dictModule.get(key, defaultValue)
    if value in valueList:
        return defaultValue
    else:
        value

def _updateApplicationMemoryData(dbConn, perfSummary):
    """
    currently not implemented and not used
    """
    pass

def _updateSystemMemoryData(dbConn, perfSummary):
    """
    currently not implemented and not used
    """
    pass
